import os
import random
import unittest

import numpy

from rdkit import DataStructs, RDConfig
from rdkit.DataManip.Metric import rdMetricMatrixCalc as rdmmc
from rdkit.SimDivFilters import rdSimDivPickers


class TestCase(unittest.TestCase):

  def setUp(self):
    self.n = 1000
    self.m = 80
    self.d = 2
    self.dataPts = numpy.zeros((self.n, self.d), 'd')
    for i in range(self.n):
      for j in range(self.d):
        self.dataPts[i, j] = random.random()
    self.dMat = rdmmc.GetEuclideanDistMat(self.dataPts)

  def test0MaxMin(self):
    pkr = rdSimDivPickers.MaxMinPicker()
    maxmin = pkr.Pick(self.dMat, self.n, self.m, (886, 112))
    self.assertEqual(maxmin[0], 886)
    self.assertEqual(maxmin[1], 112)

    def func(i, j):
      if i == j:
        return 0.0
      if i < j:
        j, i = i, j
      return self.dMat[i * (i - 1) // 2 + j]

    lmaxmin = pkr.LazyPick(func, self.n, self.m, (886, 112))
    self.assertEqual(list(lmaxmin), list(maxmin))

    lmaxmin = pkr.LazyPick(func, self.n, self.m, (886, 112), useCache=False)
    self.assertEqual(list(lmaxmin), list(maxmin))

    self.assertRaises(ValueError, lambda: pkr.Pick(self.dMat, self.n, self.m, (1012, )))
    self.assertRaises(ValueError, lambda: pkr.Pick(self.dMat, self.n, self.m, (-1, )))

    maxmin = pkr.Pick(self.dMat, self.n, self.m)
    self.assertTrue(maxmin)
    lmaxmin = pkr.LazyPick(func, self.n, self.m)
    self.assertTrue(lmaxmin)

  def test1HierarchPick(self):
    fname = os.path.join(RDConfig.RDBaseDir, 'Code', 'SimDivPickers', 'Wrap', 'test_data',
                         'points.csv')
    with open(fname) as infil:
      lines = infil.readlines()
    self.dataPts = numpy.zeros((len(lines), 2), 'd')
    labels = []
    i = 0
    for line in lines:
      tlst = line.strip().split(',')
      self.dataPts[i, 0] = float(tlst[1])
      self.dataPts[i, 1] = float(tlst[2])
      labels.append(int(tlst[3]))
      i += 1
    self.dMat = rdmmc.GetEuclideanDistMat(self.dataPts)
    pkr = rdSimDivPickers.HierarchicalClusterPicker(rdSimDivPickers.ClusterMethod.WARD)
    clusters = pkr.Cluster(self.dMat, i, 2)
    # check that each of the clusters have the same label
    for cl in clusters:
      clbl = labels[cl[0]]
      for id in cl:
        assert clbl == labels[id]
    hierarch = pkr.Pick(self.dMat, i, 2)
    self.assertEqual(tuple(hierarch), (1, 30))

  def testIssue208(self):
    sz = 10
    N = 3
    m = []
    for i in range(sz):
      for j in range(i + 1, sz):
        m.append(random.random())
    m = numpy.array(m)
    picker = rdSimDivPickers.HierarchicalClusterPicker(rdSimDivPickers.ClusterMethod.WARD)
    p1 = list(picker.Pick(m, sz, N))
    p1.sort()
    p2 = list(picker.Pick(m, sz, N))
    p2.sort()
    self.assertEqual(p1, p2)

  def testInts(self):
    """ make sure we can handle ints too """
    sz = 10
    N = 3
    m = []
    for i in range(sz):
      for j in range(i + 1, sz):
        m.append(int(100 * random.random()))
    m = numpy.array(m)
    picker = rdSimDivPickers.HierarchicalClusterPicker(rdSimDivPickers.ClusterMethod.WARD)
    p1 = list(picker.Pick(m, sz, N))
    p1.sort()
    p2 = list(picker.Pick(m, sz, N))
    p2.sort()
    self.assertEqual(p1, p2)

  def testNonUniqueCrash(self):
    from rdkit import DataStructs
    sz = 300
    nbits = 40
    nBitsToSet = int(nbits * .3)
    N = 8
    vs = []
    for i in range(sz):
      bv = DataStructs.ExplicitBitVect(nbits)
      for j in range(nBitsToSet):
        val = int(nbits * random.random())
        bv.SetBit(val)
      vs.append(bv)
      vs.append(bv)

    def taniFunc(i, j, bvs=vs):
      d = 1 - DataStructs.FingerprintSimilarity(bvs[i], bvs[j])
      return d

    picker = rdSimDivPickers.MaxMinPicker()
    mm1 = picker.LazyPick(taniFunc, len(vs), N)
    self.assertEqual(len(mm1), N)
    picker = None

    picker = rdSimDivPickers.MaxMinPicker()
    mm2 = picker.LazyBitVectorPick(vs, len(vs), N)
    self.assertEqual(len(mm2), N)

    picker = rdSimDivPickers.MaxMinPicker()
    mm3 = picker.LazyBitVectorPick(vs, len(vs), N)
    self.assertEqual(len(mm3), N)

    # we get the occasional dupe randomly,
    # make sure we don't get three dupes in a row
    self.assertTrue(tuple(mm2) != tuple(mm1)) or (tuple(mm3) != tuple(mm1))
    picker = None

    ds = []
    nvs = len(vs)
    for i in range(nvs):
      for j in range(i + 1, nvs):
        d = taniFunc(i, j)
        ds.append(d)
    m = numpy.array(ds)
    picker = rdSimDivPickers.HierarchicalClusterPicker(rdSimDivPickers.ClusterMethod.WARD)
    p1 = list(picker.Pick(m, nvs, N))

  def testBitVectorMaxMin(self):
    from rdkit import DataStructs
    sz = 100
    nbits = 200
    nBitsToSet = int(nbits * .1)
    N = 10
    vs = []
    for i in range(sz):
      bv = DataStructs.ExplicitBitVect(nbits)
      for j in range(nBitsToSet):
        val = int(nbits * random.random())
        bv.SetBit(val)
      vs.append(bv)

    def func(i, j, bvs=vs):
      d = DataStructs.TanimotoSimilarity(bvs[i], bvs[j], returnDistance=True)
      return d

    picker = rdSimDivPickers.MaxMinPicker()
    mm1 = picker.LazyPick(func, len(vs), N, seed=42)
    self.assertEqual(len(mm1), N)

    mm2 = picker.LazyPick(func, len(vs), N, useCache=False, seed=42)
    self.assertEqual(len(mm2), N)
    self.assertEqual(list(mm1), list(mm2))

    mm2 = picker.LazyBitVectorPick(vs, len(vs), N, seed=42)
    self.assertEqual(len(mm2), N)
    self.assertEqual(list(mm1), list(mm2))

    mm2 = picker.LazyBitVectorPick(vs, len(vs), N, useCache=False, seed=42)
    self.assertEqual(len(mm2), N)
    self.assertEqual(list(mm1), list(mm2))

  def testBitVectorMaxMin2(self):
    fps = [
      "11110010101000000000", "00000000000010010000", "11001010000000000001",
      "00100110101000001000", "01010110000100011001", "11000110101001000011",
      "00000000001100001111", "00011110110000001101", "00000011011110100010",
      "11000010110001000000", "00000100010000010000", "10000001000010110010",
      "00010010000000010100", "00011100100110101000", "10001001100110100000",
      "10000110100110010000", "00101110000101000000", "11011101100011100000",
      "10000110000100101000", "00101000100000010001", "01000001000010000000",
      "00101101010100000110", "10001000100110110001", "00011000010100000001",
      "00101000001000100011", "00010000100010011001", "01100001000100010001",
      "10000101000001101101", "00001000011001011000", "11110000100100100000",
      "10100110000000011010", "00110100010110010010", "00000000000001010010",
      "00100000000010100001", "11110011000010001000", "10110001010100001000",
      "00001100100110011011", "00010010100100001110", "10100101100010100010",
      "01100100010100000001", "10101110011100000000", "01011000000001000001",
      "00000011100110100010", "01100001010001001001", "00001000000001001100",
      "10011001110000000100", "10110000001001100100", "00011000000001001011",
      "11001011010001100010", "10010000000001001011", "00010000100111100000",
      "00001000001110001000", "11010000010001100110", "01101001100000111000",
      "01001000001110111000", "10000000000100010010", "11001000010010000000",
      "01010010000100110001", "00010001010100100001", "01110010000000010000",
      "10001010000011000001", "00000110000000100100", "00010000010001000000",
      "11101100011010000011", "00000010100001010001", "00010000110010000101",
      "00010001001000111001", "01000010001100100110", "00110110000000100001",
      "00100010010110110010", "01000000110011001111", "00011000001000110010",
      "01111010101000110100", "00001010000010110110", "00110011000011011010",
      "00111010111010000110", "00010011101010000011", "00000001011000010000",
      "00011011101110110000", "00010001101000000001", "00010000001010011010",
      "00000010100100100010", "00000010001011000100", "11010000000001011100",
      "00001000110101000001", "00000010000000110010", "10000000010011000001",
      "11110110100100010000", "10001111000110001001", "00100110000110000100",
      "00000100100000100100", "00110000101100010100", "00001010100000100000",
      "01011000000011000111", "00010000100001010001", "10000010100000010000",
      "00001000000000110010", "00001000101011010001", "00011110000100100000", "11001001010001010100"
    ]
    N = 5
    fps = [DataStructs.CreateFromBitString(x) for x in fps]
    picker = rdSimDivPickers.MaxMinPicker()
    mm1 = picker.LazyBitVectorPick(fps, len(fps), N, seed=42)
    self.assertEqual(len(mm1), N)
    self.assertEqual(list(mm1), [37, 1, 43, 38, 16])

    mm2 = picker.LazyBitVectorPick(fps, len(fps), N, useCache=False, seed=42)
    self.assertEqual(len(mm2), N)
    self.assertEqual(list(mm1), list(mm2))

  def testBitVectorMaxMin3(self):
    fname = os.path.join(RDConfig.RDBaseDir, 'Code', 'SimDivPickers', 'Wrap', 'test_data',
                         'chembl_cyps.head.fps')
    fps = []
    with open(fname) as infil:
      for line in infil:
        fp = DataStructs.CreateFromFPSText(line.strip())
        fps.append(fp)
    mmp = rdSimDivPickers.MaxMinPicker()
    ids = list(mmp.LazyBitVectorPick(fps, len(fps), 20, seed=42))
    self.assertEqual(ids, [
      374, 720, 690, 339, 875, 842, 404, 725, 120, 385, 115, 868, 630, 881, 516, 497, 412, 718, 869,
      407
    ])

    ids = list(
      mmp.LazyBitVectorPick(fps, len(fps), 20, firstPicks=[374, 720, 690, 339, 875], seed=42))
    self.assertEqual(ids, [
      374, 720, 690, 339, 875, 842, 404, 725, 120, 385, 115, 868, 630, 881, 516, 497, 412, 718, 869,
      407
    ])

  def testBitVectorMaxMin4(self):
    # threshold tests
    fname = os.path.join(RDConfig.RDBaseDir, 'Code', 'SimDivPickers', 'Wrap', 'test_data',
                         'chembl_cyps.head.fps')
    fps = []
    with open(fname) as infil:
      for line in infil:
        fp = DataStructs.CreateFromFPSText(line.strip())
        fps.append(fp)
    mmp = rdSimDivPickers.MaxMinPicker()
    ids, threshold = mmp.LazyBitVectorPickWithThreshold(fps, len(fps), 20, -1.0, seed=42)
    self.assertEqual(list(ids), [
      374, 720, 690, 339, 875, 842, 404, 725, 120, 385, 115, 868, 630, 881, 516, 497, 412, 718, 869,
      407
    ])

    self.assertAlmostEqual(threshold, 0.8977, 4)

    ids, threshold = mmp.LazyBitVectorPickWithThreshold(fps, len(fps), 20, 0.91, seed=42)
    self.assertEqual(list(ids), [374, 720, 690, 339, 875, 842, 404, 725, 120, 385, 115, 868, 630])
    self.assertTrue(threshold >= 0.91)

  def testBitVectorLeader1(self):
    # threshold tests
    fname = os.path.join(RDConfig.RDBaseDir, 'Code', 'SimDivPickers', 'Wrap', 'test_data',
                         'chembl_cyps.head.fps')
    fps = []
    with open(fname) as infil:
      for line in infil:
        fp = DataStructs.CreateFromFPSText(line.strip())
        fps.append(fp)
    mmp = rdSimDivPickers.LeaderPicker()
    thresh = 0.8
    ids = mmp.LazyBitVectorPick(fps, len(fps), thresh)
    self.assertEqual(len(ids), 146)
    for i in range(len(ids)):
      for j in range(i):
        self.assertGreaterEqual(1 - DataStructs.TanimotoSimilarity(fps[ids[i]], fps[ids[j]]),
                                thresh)
    thresh = 0.9
    ids = mmp.LazyBitVectorPick(fps, len(fps), thresh)
    self.assertEqual(len(ids), 14)
    for i in range(len(ids)):
      for j in range(i):
        self.assertGreaterEqual(1 - DataStructs.TanimotoSimilarity(fps[ids[i]], fps[ids[j]]),
                                thresh)

    ids = mmp.LazyBitVectorPick(fps, len(fps), thresh, pickSize=10)
    self.assertEqual(len(ids), 10)
    for i in range(len(ids)):
      for j in range(i):
        self.assertGreaterEqual(1 - DataStructs.TanimotoSimilarity(fps[ids[i]], fps[ids[j]]),
                                thresh)

  def testLazyLeader(self):
    pkr = rdSimDivPickers.LeaderPicker()

    def func(i, j):
      if i == j:
        return 0.0
      if i < j:
        j, i = i, j
      return i - j

    lres = pkr.LazyPick(func, 100, 20)
    self.assertEqual(list(lres), [0, 21, 42, 63, 84])


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2004-2005  Rational Discovery LLC
#   All Rights Reserved
#
import os
import unittest

from rdkit import RDConfig
from rdkit.DataStructs import ExplicitBitVect
from rdkit.ML.Data import DataUtils
from rdkit.ML.NaiveBayes import CrossValidate
from rdkit.ML.NaiveBayes.ClassificationModel import NaiveBayesClassifier


class TestCase(unittest.TestCase):

  def setUp(self):
    DataUtils.InitRandomNumbers((25, 25))

  def test1NaiveBayes(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'NaiveBayes', 'test_data', 'stddata.csv')
    data = DataUtils.TextFileToData(fName)

    examples = data.GetNamedData()

    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    npvals = [0] + [3] * nvars + [2]
    qBounds = [0] + [2] * nvars + [0]
    mod, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, qBounds, silent=True)
    self.assertAlmostEqual(mod._classProbs[0], 0.5000, 4)
    self.assertAlmostEqual(mod._classProbs[1], 0.5000, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][0], -0.0360, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][1], 0.114)
    self.assertAlmostEqual(mod._QBoundVals[2][0], -0.7022, 4)
    self.assertAlmostEqual(mod._QBoundVals[2][1], -0.16635, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][0], -0.3659, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][1], 0.4305, 4)
    self.assertAlmostEqual(err, 0.2121, 4)

    mod, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, qBounds, silent=True,
                                                   calcTotalError=True)
    self.assertAlmostEqual(mod._classProbs[0], 0.515151, 4)
    self.assertAlmostEqual(mod._classProbs[1], 0.484848, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][0], -0.40315, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][1], 0.114)
    self.assertAlmostEqual(mod._QBoundVals[2][0], -0.62185, 4)
    self.assertAlmostEqual(mod._QBoundVals[2][1], -0.19965, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][0], 0.4305, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][1], 0.80305, 4)
    self.assertAlmostEqual(err, 0.14563, 4)

    mod, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, qBounds, silent=True,
                                                   replacementSelection=True)
    self.assertAlmostEqual(mod._classProbs[0], 0.5131578, 4)
    self.assertAlmostEqual(mod._classProbs[1], 0.4868421, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][0], -0.036, 4)
    self.assertAlmostEqual(mod._QBoundVals[1][1], 0.93465, 4)
    self.assertAlmostEqual(mod._QBoundVals[2][0], -0.6696, 4)
    self.assertAlmostEqual(mod._QBoundVals[2][1], -0.19965, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][0], -1.06785, 4)
    self.assertAlmostEqual(mod._QBoundVals[3][1], 0.4305, 4)
    self.assertAlmostEqual(err, 0.3, 4)

  def test2NaiveBayes(self):
    fName = os.path.join(RDConfig.RDCodeDir, 'ML', 'NaiveBayes', 'test_data', 'stddata.csv')
    data = DataUtils.TextFileToData(fName)
    examples = data.GetNamedData()

    nvars = data.GetNVars()
    attrs = list(range(1, nvars + 1))
    npvals = [0] + [3] * nvars + [2]
    qBounds = [0] + [2] * nvars + [0]
    mod, err = CrossValidate.CrossValidationDriver(examples, attrs, npvals, qBounds,
                                                   mEstimateVal=20.0, silent=True)
    self.assertTrue(isinstance(mod, NaiveBayesClassifier))
    self.assertAlmostEqual(err, 0.1818, 4)

    self.assertEqual(mod.GetName(), '')
    mod.SetName('modelName')
    self.assertEqual(mod.GetName(), 'modelName')
    mod.NameModel(None)
    self.assertEqual(mod.GetName(), 'NaiveBayesClassifier')

    self.assertGreater(len(mod.GetExamples()), 0)
    self.assertGreater(len(mod.GetTrainingExamples()), 0)
    self.assertEqual(sorted(mod.GetTrainingExamples() + mod.GetExamples()), sorted(examples))

  def test3(self):
    examples = [
      ['a', 1, 0, 1, 0, 1],
      ['b', 1, 0, 0, 0, 1],
      ['c', 1, 0, 1, 0, 0],
      ['d', 0, 1, 1, 0, 0],
      ['e', 0, 1, 1, 1, 0],
    ]

    nvars = len(examples[0]) - 2
    attrs = list(range(1, nvars + 1))
    npvals = [0] + [2] * nvars + [2]
    qBounds = [0] + [0] * nvars + [0]
    mdl = CrossValidate.makeNBClassificationModel(examples, attrs, npvals, qBounds)
    nWrong = 0
    for eg in examples:
      p = mdl.ClassifyExample(eg)
      if p != eg[-1]:
        nWrong += 1
    self.assertEqual(nWrong, 1)

    bitEx = []
    for eg in examples:
      newEg = [eg[0], None, eg[-1]]
      bv = ExplicitBitVect(nvars)
      for i in range(nvars):
        if eg[i + 1]:
          bv.SetBit(i)
      newEg[1] = bv
      bitEx.append(newEg)

    attrs = list(range(nvars))
    mdl2 = CrossValidate.makeNBClassificationModel(bitEx, attrs, npvals, qBounds, useSigs=True)
    nWrong = 0
    for eg in bitEx:
      p = mdl2.ClassifyExample(eg)
      if p != eg[-1]:
        nWrong += 1
    self.assertEqual(nWrong, 1)

    # now compare:
    for i in range(len(bitEx)):
      eg = examples[i]
      p1 = mdl.ClassifyExample(eg)
      bitEg = bitEx[i]
      p2 = mdl2.ClassifyExample(bitEg)
      self.assertEqual(p1, p2)
      v1 = mdl.GetClassificationDetails()
      v2 = mdl.GetClassificationDetails()
      self.assertAlmostEqual(v1, v2, 4)

  def test4(self):
    examples = [
      ['a', 1, 0, 1, 0, 1],
      ['b', 1, 0, 0, 0, 1],
      ['c', 1, 0, 1, 0, 0],
      ['d', 0, 1, 1, 0, 0],
      ['e', 0, 1, 1, 1, 0],
    ]

    nvars = len(examples[0]) - 2
    origNVars = nvars
    nvars = 10
    npvals = [0] + [2] * nvars + [2]
    qBounds = [0] + [0] * nvars + [0]

    bitEx = []
    for eg in examples:
      newEg = [eg[0], None, eg[-1]]
      bv = ExplicitBitVect(nvars)
      for i in range(origNVars):
        if eg[i + 1]:
          bv.SetBit(i)

      # this bit will yield perfect accuracy if
      #  the attrs argument isn't being used properly:
      if eg[-1]:
        bv.SetBit(origNVars)
      newEg[1] = bv
      bitEx.append(newEg)

    attrs = list(range(origNVars))
    mdl2 = CrossValidate.makeNBClassificationModel(bitEx, attrs, npvals, qBounds, useSigs=True)
    nWrong = 0
    for eg in bitEx:
      p = mdl2.ClassifyExample(eg)
      if p != eg[-1]:
        nWrong += 1
    self.assertEqual(nWrong, 1)

  def _test5(self):  # disabled because CMIM was removed # pragma: nocover
    examples = [
      ['a', 1, 0, 1, 0, 1, 1, 0, 1],
      ['b', 1, 0, 0, 0, 1, 0, 0, 1],
      ['c', 1, 0, 1, 0, 1, 1, 0, 0],
      ['d', 0, 1, 1, 0, 1, 0, 0, 0],
      ['e', 0, 1, 1, 1, 0, 1, 0, 0],
    ]

    nvars = len(examples[0]) - 2
    npvals = [0] + [2] * nvars + [2]
    qBounds = [0] + [0] * nvars + [0]

    bitEx = []
    for eg in examples:
      newEg = [eg[0], None, eg[-1]]
      bv = ExplicitBitVect(nvars)
      for i in range(nvars):
        if eg[i + 1]:
          bv.SetBit(i)

      # this bit will yield perfect accuracy if
      #  the attrs argument isn't being used properly:
      newEg[1] = bv
      bitEx.append(newEg)

    attrs = list(range(nvars))
    mdl2 = CrossValidate.makeNBClassificationModel(bitEx, attrs, npvals, qBounds, useSigs=True,
                                                   useCMIM=2)
    nWrong = 0
    for eg in bitEx:
      p = mdl2.ClassifyExample(eg)
      if p != eg[-1]:
        nWrong += 1
    self.assertEqual(nWrong, 1)


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



import gzip
import os.path
import pickle

from rdkit import Chem, RDConfig
from rdkit.Chem import Crippen

Crippen._Init()


def runIt(inFileName, outFileName, smiCol=0, maxMols=-1, delim=','):
  inF = gzip.open(inFileName, 'r')
  outF = open(outFileName, 'wb+')
  mols = []
  nDone = 0
  for line in inF.readlines():
    if line[0] != '#':
      splitL = line.strip().split(delim)
      smi = splitL[smiCol].strip()
      print(smi)
      mol = Chem.MolFromSmiles(smi)
      if mol:
        contribs = Crippen._GetAtomContribs(mol)
        pickle.dump((smi, contribs), outF)
      nDone += 1
      if maxMols > 0 and nDone >= maxMols:
        break
  outF.close()


if __name__ == '__main__':
  inFileName = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'buildingblocks.smi.gz')
  outFileName = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data',
                             'Crippen_contribs_regress.2.pkl')
  runIt(inFileName, outFileName, smiCol=1, delim='\t', maxMols=500)



# $Id$
#
#  Copyright (C) 2004, 2005 Rational Discovery LLC
#  All Rights Reserved
#
import os
import sys
import tempfile

import Chem
from Chem import TemplateAlign
from Chem.Draw.MolDrawing import MolDrawing
from mod_python import apache
from sping.SVG.pidSVG import SVGCanvas as Canvas
from utils import cactvs


def gif(req, smiles, width=100, height=100, highlight='[]', frame=0, dblSize=0, **kwargs):
  req.content_type = 'image/gif'
  width = int(width)
  height = int(height)
  frame = int(frame)
  dblSize = int(dblSize)

  # FIX: unsafe:
  highlight = eval(highlight)
  imgD = ''
  if smiles:
    fName = tempfile.NamedTemporaryFile(suffix='.gif', delete=False).name
    cactvs.SmilesToGif(smiles, fName, (width, height), dblSize=dblSize, frame=frame)
    if os.path.exists(fName):
      imgD = open(fName, 'rb').read()
      try:
        os.unlink(fName)
      except OSError:
        pass
  return imgD


def svg(req, smiles, width=100, height=100, highlight='[]', frame=0, template='', numbers=0,
        **kwargs):
  req.content_type = 'image/svg+xml'
  width = int(width)
  height = int(height)
  frame = int(frame)
  # FIX: unsafe:
  highlight = eval(highlight)
  imgD = ''
  mol = None
  if smiles:
    mol = Chem.MolFromSmiles(smiles)
  if mol:
    if kwargs.get('kekulize', True):
      Chem.Kekulize(mol)
    if template and highlight:
      try:
        patt = Chem.MolFromSmiles(template)
        Chem.Compute2DCoords(patt)
        TemplateAlign.AlignMolToTemplate2D(mol, patt, highlight)
      except Exception:
        Chem.Compute2DCoords(mol)
    else:
      Chem.Compute2DCoords(mol)
    canvas = Canvas(size=(width, height))
    drawer = MolDrawing(canvas=canvas)
    if numbers and numbers != '0':
      drawer.includeAtomNumbers = True
      drawer.atomNumberOffset = 1
    drawer.noCarbonSymbols = 1
    drawer.AddMol(mol, highlightAtoms=highlight)
    svg = canvas._txt + '</svg>'
  else:
    svg = ''
  return svg



#  Copyright (c) 2015, Novartis Institutes for BioMedical Research Inc.
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import itertools
import os
import pickle
import sys
import time
import unittest

from rdkit import Chem, Geometry, RDConfig, rdBase
from rdkit.Chem import AllChem, rdChemReactions

test_data = [
  ("good", '''$RXN

      ISIS     052820091627

  2  1
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   -3.2730   -7.0542    0.0000 Br  0  0  0  0  0  0  0  0  0  0  0  0
   -3.9875   -7.4667    0.0000 R#  0  0  0  0  0  0  0  0  0  1  0  0
  1  2  1  0  0  0  0
V    1 halogen.bromine.aromatic
M  RGP  1   2   1
M  END
$MOL

  -ISIS-  05280916272D

  4  3  0  0  0  0  0  0  0  0999 V2000
    3.4375   -7.7917    0.0000 R#  0  0  0  0  0  0  0  0  0  2  0  0
    4.1520   -7.3792    0.0000 B   0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -6.5542    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    4.8664   -7.7917    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  2  3  1  0  0  0  0
  1  2  1  0  0  0  0
  2  4  1  0  0  0  0
V    2 boronicacid
M  RGP  1   1   2
M  END
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   11.2667   -7.3417    0.0000 R#  0  0  0  0  0  0  0  0  0  1  0  0
   11.9811   -6.9292    0.0000 R#  0  0  0  0  0  0  0  0  0  2  0  0
  1  2  1  0  0  0  0
M  RGP  2   1   1   2   2
M  END'''),
  ("bad", '''$RXN

      ISIS     052820091627

  2  1
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   -3.2730   -7.0542    0.0000 Br  0  0  0  0  0  0  0  0  0  0  0  0
   -3.9875   -7.4667    0.0000 R#  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
V    1 halogen.bromine.aromatic
M  RGP  1   2   1
M  END
$MOL

  -ISIS-  05280916272D

  4  3  0  0  0  0  0  0  0  0999 V2000
    3.4375   -7.7917    0.0000 R#  0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -7.3792    0.0000 B   0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -6.5542    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    4.8664   -7.7917    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  2  3  1  0  0  0  0
  1  2  1  0  0  0  0
  2  4  1  0  0  0  0
V    2 boronicacid
M  RGP  1   1   2
M  END
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   11.2667   -7.3417    0.0000 R#  0  0  0  0  0  0  0  0  0  0  0  0
   11.9811   -6.9292    0.0000 R#  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
M  RGP  2   1   1   2   2
M  END'''),
  # chemdraw style
  ("bad", '''$RXN

      ISIS     052820091627

  2  1
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   -3.2730   -7.0542    0.0000 Br  0  0  0  0  0  0  0  0  0  0  0  0
   -3.9875   -7.4667    0.0000 R1  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
V    1 halogen.bromine.aromatic
M  END
$MOL

  -ISIS-  05280916272D

  4  3  0  0  0  0  0  0  0  0999 V2000
    3.4375   -7.7917    0.0000 R2  0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -7.3792    0.0000 B   0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -6.5542    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    4.8664   -7.7917    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  2  3  1  0  0  0  0
  1  2  1  0  0  0  0
  2  4  1  0  0  0  0
V    2 boronicacid
M  END
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   11.2667   -7.3417    0.0000 R1  0  0  0  0  0  0  0  0  0  0  0  0
   11.9811   -6.9292    0.0000 R2  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
M  END'''),
  ("fail", '''$RXN

      ISIS     052820091627

  2  1
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   -3.2730   -7.0542    0.0000 Br  0  0  0  0  0  0  0  0  0  0  0  0
   -3.9875   -7.4667    0.0000 R1  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
V    1 halogen.bromine.aromatic
M  END
$MOL

  -ISIS-  05280916272D

  4  3  0  0  0  0  0  0  0  0999 V2000
    3.4375   -7.7917    0.0000 R3  0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -7.3792    0.0000 B   0  0  0  0  0  0  0  0  0  0  0  0
    4.1520   -6.5542    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
    4.8664   -7.7917    0.0000 O   0  0  0  0  0  0  0  0  0  0  0  0
  2  3  1  0  0  0  0
  1  2  1  0  0  0  0
  2  4  1  0  0  0  0
V    2 boronicacid
M  END
$MOL

  -ISIS-  05280916272D

  2  1  0  0  0  0  0  0  0  0999 V2000
   11.2667   -7.3417    0.0000 R1  0  0  0  0  0  0  0  0  0  0  0  0
   11.9811   -6.9292    0.0000 R2  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0  0  0  0
M  END'''),
]

unused_rlabel_in_product = """$RXN
bug.rxn
  ChemDraw06121709062D

  1  1
$MOL



  2  1  0  0  0  0  0  0  0  0999 V2000
    0.1604    0.3798    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.1604   -0.3798    0.0000 R   0  0  0  0  0  0  0  0  0  1  0  0
  1  2  1  0        0
M  END
$MOL



  2  1  0  0  0  0  0  0  0  0999 V2000
   -1.2690   -1.3345    0.0000 R   0  0  0  0  0  0  0  0  0  1  0  0
    1.2690    1.3345    0.0000 R1  0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0        0
M  END
"""

kekule_rxn = """$RXN
bug.rxn
  ChemDraw06121709062D

  1  1
$MOL

     RDKit          2D

  6  6  0  0  0  0  0  0  0  0999 V2000
    1.5000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.7500   -1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7500   -1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.5000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7500    1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.7500    1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0
  2  3  2  0
  3  4  1  0
  4  5  2  0
  5  6  1  0
  6  1  2  0
M  END
$MOL

     RDKit          2D

  6  6  0  0  0  0  0  0  0  0999 V2000
    1.5000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.7500   -1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7500   -1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -1.5000    0.0000    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
   -0.7500    1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
    0.7500    1.2990    0.0000 C   0  0  0  0  0  0  0  0  0  0  0  0
  1  2  1  0
  2  3  2  0
  3  4  1  0
  4  5  2  0
  5  6  1  0
  6  1  2  0
M  END
"""

good_res = (0, 0, 2, 1, (((0, 'halogen.bromine.aromatic'), ), ((1, 'boronicacid'), )))
bad_res = (3, 0, 2, 1, (((0, 'halogen.bromine.aromatic'), ), ((1, 'boronicacid'), )))


class TestCase(unittest.TestCase):

  def test_sanitize(self):
    for status, block in test_data:
      print("*" * 44)
      rxna = AllChem.ReactionFromRxnBlock(block)
      rxnb = AllChem.ReactionFromRxnBlock(block)
      rxna.Initialize()
      res = rdChemReactions.PreprocessReaction(rxna)
      print(AllChem.ReactionToRxnBlock(rxna))
      if status == "good":
        self.assertEqual(res, good_res)
      elif status == "bad":
        self.assertEqual(res, bad_res)
      print(">" * 44)
      rxnb.Initialize()
      try:
        rdChemReactions.SanitizeRxn(rxnb)
        res = rdChemReactions.PreprocessReaction(rxnb)
        print(AllChem.ReactionToRxnBlock(rxnb))
        self.assertEqual(res, good_res)
        assert not status == "fail"
      except Exception:
        print("$RXN Failed")
        if status == "fail":
          continue
        raise

  def test_unused_rlabel_in_product(self):
    rxn = AllChem.ReactionFromRxnBlock(unused_rlabel_in_product)
    # test was for a seg fault
    rdChemReactions.SanitizeRxn(rxn)

  def test_only_aromatize_if_possible(self):
    rxn = AllChem.ReactionFromRxnBlock(kekule_rxn)
    # test was for a seg fault
    groups = rxn.RunReactants([Chem.MolFromSmiles("c1ccccc1")])
    print(groups)
    self.assertFalse(len(groups))

    # check normal sanitization
    rdChemReactions.SanitizeRxn(rxn)
    groups = rxn.RunReactants([Chem.MolFromSmiles("c1ccccc1")])
    self.assertTrue(len(groups[0]))

    # now check adjustparams with ONLY aromatize if possible
    rxn = AllChem.ReactionFromRxnBlock(kekule_rxn)
    rdChemReactions.SanitizeRxn(rxn)

    groups = rxn.RunReactants([Chem.MolFromSmiles("c1ccccc1")])
    self.assertTrue(len(groups[0]))

  def test_github_4162(self):
    rxn = rdChemReactions.ReactionFromSmarts("[C:1](=[O:2])-[OD1].[N!H0:3]>>[C:1](=[O:2])[N:3]")
    rxn_copy = rdChemReactions.ChemicalReaction(rxn)
    rdChemReactions.SanitizeRxn(rxn)
    rdChemReactions.SanitizeRxn(rxn_copy)
    pkl = rxn.ToBinary()
    rxn_from_pickle = rdChemReactions.ChemicalReaction(pkl)
    rdChemReactions.SanitizeRxn(rxn_from_pickle)
    pkl = pickle.dumps(rxn)
    rxn_from_pickle = pickle.loads(pkl)
    rdChemReactions.SanitizeRxn(rxn_from_pickle)
    pkl = rxn_from_pickle.ToBinary()
    rxn_from_pickle = rdChemReactions.ChemicalReaction(pkl)
    rdChemReactions.SanitizeRxn(rxn_from_pickle)
    pkl = pickle.dumps(rxn_from_pickle)
    rxn_from_pickle = pickle.loads(pkl)
    rdChemReactions.SanitizeRxn(rxn_from_pickle)


if __name__ == '__main__':
  unittest.main()



# $Id$
#
#  Copyright (C) 2006  Greg Landrum
#
import os
import pickle
import sys
import unittest

from rdkit import Chem, DataStructs, RDConfig
from rdkit.Chem import MolCatalog


class TestCase(unittest.TestCase):

  def test1(self):
    cat = MolCatalog.CreateMolCatalog()
    es = []
    for smi in ('C1CCC1OC', 'C1CCC1', 'C'):
      m = Chem.MolFromSmiles(smi)
      entry = MolCatalog.MolCatalogEntry()
      entry.SetMol(m)
      self.assertTrue(entry.GetMol())
      eSmi = Chem.MolToSmiles(entry.GetMol())
      self.assertTrue(eSmi == Chem.MolToSmiles(m))
      entry.SetDescription(smi)
      self.assertTrue(entry.GetDescription() == smi)
      es.append(entry)

    v = cat.AddEntry(es[0])
    self.assertTrue(v == 0)
    self.assertTrue(cat.GetNumEntries() == 1)

    v = cat.AddEntry(es[1])
    self.assertTrue(v == 1)
    self.assertTrue(cat.GetNumEntries() == 2)

    v = cat.AddEntry(es[2])
    self.assertTrue(v == 2)
    self.assertTrue(cat.GetNumEntries() == 3)

    cat.AddEdge(0, 1)
    cat.AddEdge(0, 2)
    cat.AddEdge(1, 2)

    d = pickle.dumps(cat)
    es = None
    entry = None
    cat = None

    cat = pickle.loads(d)
    self.assertTrue(cat.GetNumEntries() == 3)
    cat = None


if __name__ == '__main__':
  unittest.main()



import unittest
from os import environ
from pathlib import Path
import re

from rdkit import Chem, DataStructs, RDConfig
from rdkit.Chem import AllChem, Descriptors
from rdkit.Chem import rdMolDescriptors as rdMD
from rdkit.Geometry import rdGeometry as rdG

haveBCUT = hasattr(rdMD, 'BCUT2D')


def feq(v1, v2, tol=1.e-4):
  return abs(v1 - v2) < tol


class TestCase(unittest.TestCase):

  def setUp(self):
    pass

  def testAtomPairTypes(self):
    params = rdMD.AtomPairsParameters
    mol = Chem.MolFromSmiles("C=C")
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0)) == rdMD.GetAtomPairAtomCode(
        mol.GetAtomWithIdx(1)))
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0)) == 1
      | (1 | 1 << params.numPiBits) << params.numBranchBits)

    mol = Chem.MolFromSmiles("C#CO")
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0)) != rdMD.GetAtomPairAtomCode(
        mol.GetAtomWithIdx(1)))
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0)) == 1
      | (2 | 1 << params.numPiBits) << params.numBranchBits)
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(1)) == 2
      | (2 | 1 << params.numPiBits) << params.numBranchBits)
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(2)) == 1
      | (0 | 3 << params.numPiBits) << params.numBranchBits)
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(1), 1) == 1
      | (2 | 1 << params.numPiBits) << params.numBranchBits)
    self.assertTrue(
      rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(1), 2) == 0
      | (2 | 1 << params.numPiBits) << params.numBranchBits)

  def testAtomPairTypesChirality(self):
    mols = [Chem.MolFromSmiles(x) for x in ("CC(F)Cl", "C[C@@H](F)Cl", "C[C@H](F)Cl")]
    self.assertEqual(rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1)),
                     rdMD.GetAtomPairAtomCode(mols[1].GetAtomWithIdx(1)))
    self.assertEqual(rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1)),
                     rdMD.GetAtomPairAtomCode(mols[2].GetAtomWithIdx(1)))
    self.assertEqual(rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1), includeChirality=True),
                     rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1)))
    self.assertNotEqual(rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1), includeChirality=True),
                        rdMD.GetAtomPairAtomCode(mols[1].GetAtomWithIdx(1), includeChirality=True))
    self.assertNotEqual(rdMD.GetAtomPairAtomCode(mols[0].GetAtomWithIdx(1), includeChirality=True),
                        rdMD.GetAtomPairAtomCode(mols[2].GetAtomWithIdx(1), includeChirality=True))
    self.assertNotEqual(rdMD.GetAtomPairAtomCode(mols[1].GetAtomWithIdx(1), includeChirality=True),
                        rdMD.GetAtomPairAtomCode(mols[2].GetAtomWithIdx(1), includeChirality=True))

    fps = [rdMD.GetAtomPairFingerprint(x) for x in mols]
    chiralFps = [rdMD.GetAtomPairFingerprint(x, includeChirality=True) for x in mols]
    for mol, fp, cfp in zip(mols, fps, chiralFps):
      ac0 = rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0))
      ac1 = rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(1))
      self.assertTrue(rdMD.GetAtomPairCode(ac0, ac1, 1) in fp.GetNonzeroElements())
      ac0 = rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(0), includeChirality=True)
      ac1 = rdMD.GetAtomPairAtomCode(mol.GetAtomWithIdx(1), includeChirality=True)
      self.assertFalse(
        rdMD.GetAtomPairCode(ac0, ac1, 1, includeChirality=True) in fp.GetNonzeroElements())
      self.assertTrue(
        rdMD.GetAtomPairCode(ac0, ac1, 1, includeChirality=True) in cfp.GetNonzeroElements())

  def testAtomPairs(self):
    m = Chem.MolFromSmiles('CCC')
    fp1 = rdMD.GetAtomPairFingerprint(m)
    fp2 = rdMD.GetAtomPairFingerprint(m, minLength=1, maxLength=2)
    nz1 = fp1.GetNonzeroElements()
    self.assertEqual(len(nz1), 2)
    nz2 = fp2.GetNonzeroElements()
    self.assertEqual(len(nz2), 2)

    fp2 = rdMD.GetAtomPairFingerprint(m, minLength=1, maxLength=1)
    nz2 = fp2.GetNonzeroElements()
    self.assertEqual(len(nz2), 1)

  def testHashedAtomPairs(self):
    m = Chem.MolFromSmiles('c1ccccc1')
    fp1 = rdMD.GetHashedAtomPairFingerprint(m, 2048)
    fp2 = rdMD.GetHashedAtomPairFingerprint(m, 2048, 1, 3)
    self.assertTrue(fp1 == fp2)
    fp2 = rdMD.GetHashedAtomPairFingerprint(m, 2048, 1, 2)
    sim = DataStructs.DiceSimilarity(fp1, fp2)
    self.assertTrue(sim > 0.0 and sim < 1.0)

    m = Chem.MolFromSmiles('c1ccccn1')
    fp2 = rdMD.GetHashedAtomPairFingerprint(m, 2048)
    sim = DataStructs.DiceSimilarity(fp1, fp2)
    self.assertTrue(sim > 0.0 and sim < 1.0)

    m = Chem.MolFromSmiles('c1ccccc1')
    fp1 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m, 2048)
    m = Chem.MolFromSmiles('c1ccccn1')
    fp2 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m, 2048)
    sim = DataStructs.DiceSimilarity(fp1, fp2)
    self.assertTrue(sim > 0.0 and sim < 1.0)

  def testRootedAtomPairs(self):
    m = Chem.MolFromSmiles('Oc1ccccc1')
    fp1 = rdMD.GetAtomPairFingerprint(m)
    fp2 = rdMD.GetAtomPairFingerprint(m, fromAtoms=(0, ))
    nz1 = fp1.GetNonzeroElements()
    nz2 = fp2.GetNonzeroElements()
    for k, v in nz2.items():
      self.assertTrue(v <= nz1[k])

  def testTopologicalTorsions(self):
    mol = Chem.MolFromSmiles("CC")
    fp = rdMD.GetTopologicalTorsionFingerprint(mol)
    self.assertTrue(fp.GetTotalVal() == 0)

    mol = Chem.MolFromSmiles("CCCC")
    fp = rdMD.GetTopologicalTorsionFingerprint(mol)
    self.assertTrue(fp.GetTotalVal() == 1)
    fp = rdMD.GetTopologicalTorsionFingerprint(mol, 3)
    self.assertTrue(fp.GetTotalVal() == 2)

    mol = Chem.MolFromSmiles("CCCO")
    fp = rdMD.GetTopologicalTorsionFingerprint(mol)
    self.assertTrue(fp.GetTotalVal() == 1)
    fp = rdMD.GetTopologicalTorsionFingerprint(mol, 3)
    self.assertTrue(fp.GetTotalVal() == 2)

    mol = Chem.MolFromSmiles("CCCCCCCCCCC")
    fp = rdMD.GetTopologicalTorsionFingerprint(mol, 7)
    self.assertRaises(ValueError, lambda: rdMD.GetTopologicalTorsionFingerprint(mol, 8))

  def testHashedTopologicalTorsions(self):
    mol = Chem.MolFromSmiles("c1ncccc1")
    fp1 = rdMD.GetHashedTopologicalTorsionFingerprint(mol)
    mol = Chem.MolFromSmiles("n1ccccc1")
    fp2 = rdMD.GetHashedTopologicalTorsionFingerprint(mol)
    self.assertEqual(DataStructs.DiceSimilarity(fp1, fp2), 1.0)

  def testRootedTorsions(self):
    m = Chem.MolFromSmiles('Oc1ccccc1')
    fp1 = rdMD.GetTopologicalTorsionFingerprint(m)
    fp2 = rdMD.GetTopologicalTorsionFingerprint(m, fromAtoms=(0, ))
    nz1 = fp1.GetNonzeroElements()
    nz2 = fp2.GetNonzeroElements()
    for k, v in nz2.items():
      self.assertTrue(v <= nz1[k])

    m = Chem.MolFromSmiles('COCC')
    fp1 = rdMD.GetTopologicalTorsionFingerprint(m)
    self.assertEqual(len(fp1.GetNonzeroElements()), 1)
    fp1 = rdMD.GetTopologicalTorsionFingerprint(m, fromAtoms=(0, ))
    self.assertEqual(len(fp1.GetNonzeroElements()), 1)
    fp1 = rdMD.GetTopologicalTorsionFingerprint(m, fromAtoms=(1, ))
    self.assertEqual(len(fp1.GetNonzeroElements()), 0)

  def testMorganFingerprints(self):
    mol = Chem.MolFromSmiles('CC(F)(Cl)C(F)(Cl)C')
    fp = rdMD.GetMorganFingerprint(mol, 0)
    self.assertTrue(len(fp.GetNonzeroElements()) == 4)

    mol = Chem.MolFromSmiles('CC')
    fp = rdMD.GetMorganFingerprint(mol, 0)
    self.assertTrue(len(fp.GetNonzeroElements()) == 1)
    self.assertTrue(list(fp.GetNonzeroElements().values())[0] == 2)
    fp = rdMD.GetMorganFingerprint(mol, 0, useCounts=False)
    self.assertTrue(len(fp.GetNonzeroElements()) == 1)
    self.assertTrue(list(fp.GetNonzeroElements().values())[0] == 1)

    mol = Chem.MolFromSmiles('CC(F)(Cl)C(F)(Cl)C')
    fp = rdMD.GetHashedMorganFingerprint(mol, 0)
    self.assertTrue(len(fp.GetNonzeroElements()) == 4)
    fp = rdMD.GetMorganFingerprint(mol, 1)
    self.assertTrue(len(fp.GetNonzeroElements()) == 8)
    fp = rdMD.GetHashedMorganFingerprint(mol, 1)
    self.assertTrue(len(fp.GetNonzeroElements()) == 8)
    fp = rdMD.GetMorganFingerprint(mol, 2)
    self.assertTrue(len(fp.GetNonzeroElements()) == 9)

    mol = Chem.MolFromSmiles('CC(F)(Cl)[C@](F)(Cl)C')
    fp = rdMD.GetMorganFingerprint(mol, 0)
    self.assertTrue(len(fp.GetNonzeroElements()) == 4)
    fp = rdMD.GetMorganFingerprint(mol, 1)
    self.assertTrue(len(fp.GetNonzeroElements()) == 8)
    fp = rdMD.GetMorganFingerprint(mol, 2)
    self.assertTrue(len(fp.GetNonzeroElements()) == 9)
    fp = rdMD.GetMorganFingerprint(mol, 0, useChirality=True)
    self.assertTrue(len(fp.GetNonzeroElements()) == 4)
    fp = rdMD.GetMorganFingerprint(mol, 1, useChirality=True)
    self.assertTrue(len(fp.GetNonzeroElements()) == 9)
    fp = rdMD.GetMorganFingerprint(mol, 2, useChirality=True)
    self.assertTrue(len(fp.GetNonzeroElements()) == 10)

    mol = Chem.MolFromSmiles('CCCCC')
    fp = rdMD.GetMorganFingerprint(mol, 0, fromAtoms=(0, ))
    self.assertTrue(len(fp.GetNonzeroElements()) == 1)

    mol = Chem.MolFromSmiles('CC1CC1')
    vs1 = rdMD.GetConnectivityInvariants(mol)
    self.assertEqual(len(vs1), mol.GetNumAtoms())
    fp1 = rdMD.GetMorganFingerprint(mol, 2, invariants=vs1)
    fp2 = rdMD.GetMorganFingerprint(mol, 2)
    self.assertEqual(fp1, fp2)

    vs2 = rdMD.GetConnectivityInvariants(mol, False)
    self.assertEqual(len(vs2), mol.GetNumAtoms())
    self.assertNotEqual(vs1, vs2)
    fp1 = rdMD.GetMorganFingerprint(mol, 2, invariants=vs2)
    self.assertNotEqual(fp1, fp2)

    mol = Chem.MolFromSmiles('Cc1ccccc1')
    vs1 = rdMD.GetFeatureInvariants(mol)
    self.assertEqual(len(vs1), mol.GetNumAtoms())
    self.assertEqual(vs1[0], 0)
    self.assertNotEqual(vs1[1], 0)
    self.assertEqual(vs1[1], vs1[2])
    self.assertEqual(vs1[1], vs1[3])
    self.assertEqual(vs1[1], vs1[4])

    mol = Chem.MolFromSmiles('FCCCl')
    vs1 = rdMD.GetFeatureInvariants(mol)
    self.assertEqual(len(vs1), mol.GetNumAtoms())
    self.assertEqual(vs1[1], 0)
    self.assertEqual(vs1[2], 0)
    self.assertNotEqual(vs1[0], 0)
    self.assertEqual(vs1[0], vs1[3])

    fp1 = rdMD.GetMorganFingerprint(mol, 0, invariants=vs1)
    fp2 = rdMD.GetMorganFingerprint(mol, 0, useFeatures=True)
    self.assertEqual(fp1, fp2)

    mol = Chem.MolFromSmiles('CCCC(N)(O)')
    info = {}
    fp = rdMD.GetMorganFingerprint(mol, 2, bitInfo=info, includeRedundantEnvironments=True)
    self.assertEqual(len(info), 17)

    info1 = {}
    fp1 = rdMD.GetMorganFingerprint(mol, 2, bitInfo=info1, includeRedundantEnvironments=False)
    self.assertEqual(len(info1), 14)

    info2 = {}
    fp1 = rdMD.GetMorganFingerprint(mol, 2, bitInfo=info2)
    self.assertEqual(len(info2), 14)

  def testCrippen(self):
    mol = Chem.MolFromSmiles("n1ccccc1CO")
    contribs = rdMD._CalcCrippenContribs(mol)
    self.assertEqual(len(contribs), mol.GetNumAtoms())

    ts = [0] * mol.GetNumAtoms()
    contribs = rdMD._CalcCrippenContribs(mol, force=True, atomTypes=ts)
    self.assertEqual(ts, [59, 25, 25, 25, 25, 28, 17, 69])

    ls = [''] * mol.GetNumAtoms()
    contribs = rdMD._CalcCrippenContribs(mol, force=True, atomTypeLabels=ls)
    self.assertEqual(ls, ['N11', 'C18', 'C18', 'C18', 'C18', 'C21', 'C10', 'O2'])

  def testUSR(self):
    mol = Chem.MolFromSmiles("CC")
    AllChem.Compute2DCoords(mol)
    self.assertRaises(ValueError, lambda: rdMD.GetUSR(mol))
    mol = Chem.MolFromSmiles("C1CCCCC1")
    mol = Chem.AddHs(mol)
    self.assertRaises(ValueError, lambda: rdMD.GetUSR(mol))
    AllChem.Compute2DCoords(mol)
    usr = rdMD.GetUSR(mol)
    self.assertEqual(len(usr), 12)

    self.assertRaises(ValueError, lambda: rdMD.GetUSRDistributions([]))

    conf = mol.GetConformer()
    coords = [conf.GetAtomPosition(i) for i in range(mol.GetNumAtoms())]
    dist = rdMD.GetUSRDistributions(coords)
    self.assertEqual(len(dist), 4)
    self.assertEqual(len(dist[0]), mol.GetNumAtoms())
    self.assertRaises(ValueError, lambda: rdMD.GetUSRFromDistributions([]))
    usr2 = rdMD.GetUSRFromDistributions(dist)
    self.assertEqual(usr, usr2)

    self.assertRaises(ValueError, lambda: rdMD.GetUSRDistributionsFromPoints(coords, []))
    p = []
    dist = rdMD.GetUSRDistributions(coords, p)
    self.assertRaises(ValueError, lambda: rdMD.GetUSRDistributionsFromPoints([], p))
    dist2 = rdMD.GetUSRDistributionsFromPoints(coords, p)
    usr2 = rdMD.GetUSRFromDistributions(dist2)
    self.assertEqual(usr, usr2)

    mol2 = Chem.MolFromSmiles("C1CCCCC1")
    mol2 = Chem.AddHs(mol2)
    AllChem.Compute2DCoords(mol2)
    usr2 = rdMD.GetUSR(mol2)
    self.assertRaises(ValueError, lambda: rdMD.GetUSRScore(usr, usr2[:2]))
    self.assertEqual(rdMD.GetUSRScore(usr, usr2), 1.0)

    m1 = [4.44, 2.98, 1.04, 4.55, 4.70, 0.23, 8.30, 16.69, -22.97, 7.37, 15.64, 0.51]
    m2 = [4.39, 3.11, 1.36, 4.50, 4.44, 0.09, 8.34, 16.78, -23.20, 7.15, 16.52, 0.13]
    self.assertAlmostEqual(rdMD.GetUSRScore(m1, m2), 0.812, 2)

  def testUSRCAT(self):
    mol = Chem.MolFromSmiles("CC")
    AllChem.Compute2DCoords(mol)
    self.assertRaises(ValueError, lambda: rdMD.GetUSRCAT(mol))
    mol = Chem.MolFromSmiles("C1CCCCC1")
    mol = Chem.AddHs(mol)
    self.assertRaises(ValueError, lambda: rdMD.GetUSRCAT(mol))
    AllChem.Compute2DCoords(mol)
    usr = rdMD.GetUSRCAT(mol)
    self.assertEqual(len(usr), 60)
    self.assertRaises(ValueError, lambda: rdMD.GetUSRCAT(mol, atomSelections=[]))
    atoms = [[1, 2, 3, 4, 5, 6], []]
    usr2 = rdMD.GetUSRCAT(mol, atomSelections=atoms)
    self.assertEqual(len(usr2), 36)
    atoms = [[1, 2, 3, 4, 5, 6], [], [], []]
    usr2 = rdMD.GetUSRCAT(mol, atomSelections=atoms)
    self.assertEqual(len(usr2), 60)
    self.assertEqual(rdMD.GetUSRScore(usr, usr2, weights=[1.0, 1.0, 1.0, 1.0, 1.0]), 1.0)

  def testMolWt(self):
    mol = Chem.MolFromSmiles("C")
    amw = rdMD._CalcMolWt(mol)
    self.assertTrue(feq(amw, 16.043, .001))
    amw = rdMD._CalcMolWt(mol, True)
    self.assertTrue(feq(amw, 12.011, .001))
    mol2 = Chem.AddHs(mol)
    amw = rdMD._CalcMolWt(mol2)
    self.assertTrue(feq(amw, 16.043, .001))
    amw = rdMD._CalcMolWt(mol2, True)
    self.assertTrue(feq(amw, 12.011, .001))

    mol = Chem.MolFromSmiles("C")
    amw = rdMD.CalcExactMolWt(mol)
    self.assertTrue(feq(amw, 16.031, .001))

  def testPairValues(self):
    import base64
    testD = (
      ('CCCO',
       b'AQAAAAQAAAAAAIAABgAAACGECAABAAAAIoQIAAEAAABBhAgAAQAAACNEGAABAAAAQUQYAAEAAABC\nRBgAAQAAAA==\n'
       ),
      ('CNc1ccco1',
       b'AQAAAAQAAAAAAIAAEAAAACOECgABAAAAJIQKAAIAAABBhQoAAgAAAEKFCgABAAAAIsQKAAEAAABB\nxQoAAQAAAELFCgACAAAAIYQQAAEAAABChRAAAQAAAEOFEAACAAAAYYUQAAEAAAAjhBoAAQAAAEGF\nGgABAAAAQoUaAAIAAABhhRoAAQAAAEKIGgABAAAA\n'
       ),
    )
    for smi, txt in testD:
      pkl = base64.decodebytes(txt)
      fp = rdMD.GetAtomPairFingerprint(Chem.MolFromSmiles(smi))
      fp2 = DataStructs.IntSparseIntVect(pkl)
      self.assertEqual(DataStructs.DiceSimilarity(fp, fp2), 1.0)
      self.assertEqual(fp, fp2)

  def testTorsionValues(self):
    import base64
    testD = (
      ('CCCO', b'AQAAAAgAAAD/////DwAAAAEAAAAAAAAAIECAAAMAAAABAAAA\n'),
      ('CNc1ccco1',
       b'AQAAAAgAAAD/////DwAAAAkAAAAAAAAAIICkSAEAAAABAAAAKVKgSQEAAAABAAAAKVCgUAEAAAAB\nAAAAKVCgUQEAAAABAAAAKVCkCAIAAAABAAAAKdCkCAIAAAABAAAAKVCgSAMAAAABAAAAKVCkSAMA\nAAABAAAAIICkSAMAAAABAAAA\n'
       ),
    )
    for smi, txt in testD:
      pkl = base64.decodebytes(txt)
      fp = rdMD.GetTopologicalTorsionFingerprint(Chem.MolFromSmiles(smi))
      fp2 = DataStructs.LongSparseIntVect(pkl)
      self.assertEqual(DataStructs.DiceSimilarity(fp, fp2), 1.0)
      self.assertEqual(fp, fp2)

  def testAtomPairOptions(self):
    m1 = Chem.MolFromSmiles('c1ccccc1')
    m2 = Chem.MolFromSmiles('c1ccccn1')

    fp1 = rdMD.GetAtomPairFingerprint(m1)
    fp2 = rdMD.GetAtomPairFingerprint(m2)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetAtomPairFingerprint(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetAtomPairFingerprint(m2, atomInvariants=[1] * 6)
    self.assertEqual(fp1, fp2)

    fp1 = rdMD.GetAtomPairFingerprint(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetAtomPairFingerprint(m2, atomInvariants=[2] * 6)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m1)
    fp2 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m2)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m2, atomInvariants=[1] * 6)
    self.assertEqual(fp1, fp2)

    fp1 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetHashedAtomPairFingerprintAsBitVect(m2, atomInvariants=[2] * 6)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetTopologicalTorsionFingerprint(m1)
    fp2 = rdMD.GetTopologicalTorsionFingerprint(m2)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetTopologicalTorsionFingerprint(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetTopologicalTorsionFingerprint(m2, atomInvariants=[1] * 6)
    self.assertEqual(fp1, fp2)

    fp1 = rdMD.GetTopologicalTorsionFingerprint(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetTopologicalTorsionFingerprint(m2, atomInvariants=[2] * 6)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m1)
    fp2 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m2)
    self.assertNotEqual(fp1, fp2)

    fp1 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m2, atomInvariants=[1] * 6)
    self.assertEqual(fp1, fp2)

    fp1 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m1, atomInvariants=[1] * 6)
    fp2 = rdMD.GetHashedTopologicalTorsionFingerprintAsBitVect(m2, atomInvariants=[2] * 6)
    self.assertNotEqual(fp1, fp2)

  def testMolFormula(self):
    m = Chem.MolFromSmiles("[2H]C([3H])O")
    formula = rdMD.CalcMolFormula(m)
    self.assertEqual(formula, 'CH4O')
    formula = rdMD.CalcMolFormula(m, separateIsotopes=True)
    self.assertEqual(formula, 'CH2DTO')
    formula = rdMD.CalcMolFormula(m, separateIsotopes=True, abbreviateHIsotopes=False)
    self.assertEqual(formula, 'CH2[2H][3H]O')

    m = Chem.MolFromSmiles("[2H][13CH2]CO")
    formula = rdMD.CalcMolFormula(m)
    self.assertEqual(formula, 'C2H6O')
    formula = rdMD.CalcMolFormula(m, separateIsotopes=True)
    self.assertEqual(formula, 'C[13C]H5DO')

  def testSpiroAndBridgeheads(self):
    m = Chem.MolFromSmiles("C1CC2CCC1CC2")
    self.assertEqual(rdMD.CalcNumSpiroAtoms(m), 0)
    sa = []
    self.assertEqual(rdMD.CalcNumSpiroAtoms(m, atoms=sa), 0)
    self.assertEqual(len(sa), 0)

    self.assertEqual(rdMD.CalcNumBridgeheadAtoms(m), 2)
    sa = []
    self.assertEqual(rdMD.CalcNumBridgeheadAtoms(m, atoms=sa), 2)
    self.assertEqual(len(sa), 2)
    self.assertEqual(sorted(sa), [2, 5])

    m = Chem.MolFromSmiles("C1CCC2(C1)CC1CCC2CC1")
    self.assertEqual(rdMD.CalcNumSpiroAtoms(m), 1)
    sa = []
    self.assertEqual(rdMD.CalcNumSpiroAtoms(m, atoms=sa), 1)
    self.assertEqual(len(sa), 1)
    self.assertEqual(sorted(sa), [3])

    self.assertEqual(rdMD.CalcNumBridgeheadAtoms(m), 2)
    sa = []
    self.assertEqual(rdMD.CalcNumBridgeheadAtoms(m, atoms=sa), 2)
    self.assertEqual(len(sa), 2)
    self.assertEqual(sorted(sa), [6, 9])

  def testNumRotatableBonds(self):
    for s in [
        "C1CC1CC",
        "CCNC(=O)NCC",
        'Cc1cccc(C)c1c1c(C)cccc1C',
        'CCc1cccc(C)c1c1c(C)cccc1CC',
        'Cc1cccc(C)c1c1c(C)nccc1C',
        'Cc1cccc(C)c1c1c(C)cccc1',
        'CCO',
    ]:
      m = Chem.MolFromSmiles(s)

      v1 = rdMD.CalcNumRotatableBonds(m)

      v2 = rdMD.CalcNumRotatableBonds(m, False)
      v3 = rdMD.CalcNumRotatableBonds(m, True)

      v4 = rdMD.CalcNumRotatableBonds(m, rdMD.NumRotatableBondsOptions.Default)
      v5 = rdMD.CalcNumRotatableBonds(m, rdMD.NumRotatableBondsOptions.NonStrict)
      v6 = rdMD.CalcNumRotatableBonds(m, rdMD.NumRotatableBondsOptions.Strict)
      v7 = rdMD.CalcNumRotatableBonds(m, rdMD.NumRotatableBondsOptions.StrictLinkages)

      self.assertEqual(v1, v4)
      self.assertEqual(v2, v5)
      self.assertEqual(v3, v6)

  def testProperties(self):
    props = rdMD.Properties()
    names = list(props.GetAvailableProperties())
    self.assertEqual(names, list(props.GetPropertyNames()))
    m = Chem.MolFromSmiles("C1CC1CC")
    results = props.ComputeProperties(m)

    for i, name in enumerate(names):
      props = rdMD.Properties([name])
      res = props.ComputeProperties(m)
      self.assertEqual(len(res), 1)
      self.assertEqual(res[0], results[i])
      self.assertEqual(props.GetPropertyNames()[0], names[i])
      self.assertEqual(len(props.GetPropertyNames()), 1)

    try:
      props = rdMD.Properties([1, 2, 3])
      self.assertEqual("should not get here", "but did")
    except TypeError:
      pass

    try:
      props = rdMD.Properties(["property that doesn't exist"])
      self.assertEqual("should not get here", "but did")
    except KeyError:
      pass

  def testPythonDescriptorFunctor(self):

    class NumAtoms(Descriptors.PropertyFunctor):

      def __init__(self):
        Descriptors.PropertyFunctor.__init__(self, "NumAtoms", "1.0.0")

      def __call__(self, mol):
        return mol.GetNumAtoms()

    numAtoms = NumAtoms()
    rdMD.Properties.RegisterProperty(numAtoms)
    props = rdMD.Properties(["NumAtoms"])
    self.assertEqual(1, props.ComputeProperties(Chem.MolFromSmiles("C"))[0])

    self.assertTrue("NumAtoms" in rdMD.Properties.GetAvailableProperties())
    # check memory
    del numAtoms
    self.assertEqual(1, props.ComputeProperties(Chem.MolFromSmiles("C"))[0])
    self.assertTrue("NumAtoms" in rdMD.Properties.GetAvailableProperties())

    m = Chem.MolFromSmiles("c1ccccc1")
    properties = rdMD.Properties()
    for name, value in zip(properties.GetPropertyNames(), properties.ComputeProperties(m)):
      print(name, value)

    properties = rdMD.Properties(['exactmw', 'lipinskiHBA'])
    for name, value in zip(properties.GetPropertyNames(), properties.ComputeProperties(m)):
      print(name, value)

  def testPropertyRanges(self):
    query = rdMD.MakePropertyRangeQuery("exactmw", 0, 1000)
    self.assertTrue(query.Match(Chem.MolFromSmiles("C")))

    query = rdMD.MakePropertyRangeQuery("exactmw", 1000, 10000)
    self.assertFalse(query.Match(Chem.MolFromSmiles("C")))

  def testNumStereoCenters(self):
    m = Chem.MolFromSmiles('CC(F)(Cl)[C@H](Cl)Br')
    self.assertEqual(rdMD.CalcNumAtomStereoCenters(m), 2)
    self.assertEqual(rdMD.CalcNumUnspecifiedAtomStereoCenters(m), 1)
    # Tests from Berend Huisman:
    for (smiles, expected) in (
      ("C", 0),
      ("c1ccccc1", 0),
      ("CC(Cl)Br", 1),
      ("CCC(C)C(Cl)Br", 2),
      ("CCC(C(Cl)Br)C(F)I", 3),
      ("[H][C@](F)(I)C(CC)C(Cl)Br", 3),
      ("[H][C@](F)(I)[C@@]([H])(CC)C(Cl)Br", 3),
    ):
      mol = Chem.MolFromSmiles(smiles)
      actual = len(Chem.FindMolChiralCenters(mol, includeUnassigned=True))
      self.assertEqual(rdMD.CalcNumAtomStereoCenters(mol), expected)
    for (smiles, expected) in (
      ("C", 0),
      ("c1ccccc1", 0),
      ("CC(Cl)Br", 1),
      ("CCC(C)C(Cl)Br", 2),
      ("CCC(C(Cl)Br)C(F)I", 3),
      ("[H][C@](F)(I)C(CC)C(Cl)Br", 2),
      ("[H][C@](F)(I)[C@@]([H])(CC)C(Cl)Br", 1),
    ):
      mol = Chem.MolFromSmiles(smiles)
      actual = sum(1 for x in Chem.FindMolChiralCenters(mol, includeUnassigned=True) if x[1] == '?')
      self.assertEqual(actual, expected)
      self.assertEqual(rdMD.CalcNumUnspecifiedAtomStereoCenters(mol), expected)

  def testGithub1749(self):
    mol = Chem.MolFromSmiles("c1ccccc1O")
    self.assertRaises(ValueError,
                      lambda: rdMD.GetMorganFingerprintAsBitVect(mol, 2, fromAtoms=[10]))

  def testCustomVSA(self):
    mol = Chem.MolFromSmiles("c1ccccc1O")
    peoe_vsa = rdMD.PEOE_VSA_(mol)
    AllChem.ComputeGasteigerCharges(mol)
    bins = [-.3, -.25, -.20, -.15, -.10, -.05, 0, .05, .10, .15, .20, .25, .30]
    custom_vsa = rdMD.CustomProp_VSA_(mol, customPropName='_GasteigerCharge', bins=bins)
    for p, c in zip(peoe_vsa, custom_vsa):
      self.assertTrue(feq(p, c, .001))

  def testGithub1973(self):

    smiles = ("c1ccccc1S", "c1cscc1", "CC(=S)C", "CSC", "CS(=O)C", "CP(C)C", "CP=O", "CP(C)(C)=O",
              "C[PH](C)=O")
    orig_tpsa = (0, 0, 0, 0, 17.07, 0.0, 17.07, 17.07, 17.07)
    new_tpsa = (38.8, 28.24, 32.09, 25.30, 36.28, 13.59, 51.21, 26.88, 40.54)
    for i, smi in enumerate(smiles):
      mol = Chem.MolFromSmiles(smi)
      oTPSA = rdMD.CalcTPSA(mol)
      self.assertAlmostEqual(oTPSA, orig_tpsa[i], 2)
      nTPSA = rdMD.CalcTPSA(mol, force=True, includeSandP=True)
      self.assertAlmostEqual(nTPSA, new_tpsa[i], 2)

  def testGithub1761(self):
    mol = Chem.MolFromSmiles('CC(F)(Cl)C(F)(Cl)C')
    self.assertRaises(OverflowError, lambda: rdMD.GetMorganFingerprint(mol, -1))
    self.assertRaises(OverflowError, lambda: rdMD.GetHashedMorganFingerprint(mol, 0, -1))
    self.assertRaises(ValueError, lambda: rdMD.GetHashedMorganFingerprint(mol, 0, 0))

  @unittest.skipIf(not haveBCUT, "BCUT descriptors not present")
  def testBCUT(self):
    smiles = ("c1ccccc1S", "c1cscc1", "CC(=S)C", "CSC", "CS(=O)C", "CP(C)C", "CP=O", "CP(C)(C)=O",
              "C[PH](C)=O")
    expected = [[
      32.11691025659743, 10.3711255714102, 1.7258015589384423, -1.813025067747632,
      2.0032623406582895, -1.5648237280483932, 7.798895708674262, 1.480463460412681
    ],
                [
                  32.133385673916855, 10.690498814349603, 1.5812862454321042, -1.3924158848795094,
                  1.9109090827066813, -1.1624011856503906, 7.073072952311218, 2.0294988143496093
                ],
                [
                  32.09203038492705, 10.585642854940236, 1.5531725323483636, -1.6095571920399787,
                  1.6095355058255407, -1.6264025009240035, 7.798379459554676, 1.8522004089765778
                ],
                [
                  32.16623472793675, 11.912765272063261, 1.3229624149945334, -1.5095023144073105,
                  1.699644712375421, -1.2539447123754206, 7.974132068991219, 2.370867931008784
                ],
                [
                  32.19696040763111, 11.912196087574864, 1.571634336328628, -1.5986639964786287,
                  1.4708186703193278, -1.7011978973115978, 7.829987043024966, 0.6900118228151252
                ],
                [
                  31.130920829765557, 11.856079170234391, 1.652795743173261, -1.8120831465729346,
                  2.1415709560249625, -1.4818709560249614, 7.546153955380794, 2.12884604461921
                ],
                [
                  31.059700934072303, 11.958086319851176, 1.271920282698931, -1.2044337292760101,
                  1.6734005705339297, -0.9875941256320955, 7.220415332975225, 0.601201124030627
                ],
                [
                  31.163638790374048, 11.855143433013291, 1.8876168729214116, -1.862911092419536,
                  2.279553713445213, -1.6070814330824321, 7.610231826377116, 0.5859042224902173
                ],
                [
                  31.111807418763107, 11.906469225329653, 1.5848247701762241, -1.5909887821270712,
                  2.0038391834918317, -1.3277290609201042, 7.422700514979134, 0.5941465179281992
                ]]
    for i, smi in enumerate(smiles):
      mol = Chem.MolFromSmiles(smi)
      res = rdMD.BCUT2D(mol)
      self.assertEqual(len(res), 8)
      for rv, ev in zip(res, expected[i]):
        self.assertAlmostEqual(rv, ev)
      # print(list(res))  # - expected[i])
      # self.assertAlmostEqual(list(res), expected[i])

  @unittest.skipIf(not haveBCUT, "BCUT descriptors not present")
  def testBCUTUserProps(self):
    m = Chem.MolFromSmiles("CCCCCC")
    props = []
    for a in m.GetAtoms():
      a.SetDoubleProp("prop", a.GetIdx() + 1)
      props.append(float(a.GetIdx() + 1))

    bcut1 = rdMD.BCUT2D(m, "prop")
    m = Chem.MolFromSmiles("CCCCCC")
    bcut2 = rdMD.BCUT2D(m, props)
    bcut3 = rdMD.BCUT2D(m, tuple(props))

    # might need feq
    self.assertEqual(list(bcut1), list(bcut2))
    self.assertEqual(list(bcut3), list(bcut2))

    props.append(0.0)
    try:
      bcut2 = rdMD.BCUT2D(m, props)
      self.assertTrue(0, "Failed to handle bad prop size")
    except RuntimeError as e:
      self.assertTrue("tom_props.size() == num_atoms" in str(e))

    try:
      bcut2 = rdMD.BCUT2D(m, "property not existing on the atom")
      self.assertTrue(0, "Failed to handle not existing properties")
    except KeyError as e:
      self.assertEqual(e.args, ("property not existing on the atom", ))

    for atom in m.GetAtoms():
      atom.SetProp("bad_prop", "not a double")
      break

    try:
      bcut2 = rdMD.BCUT2D(m, "bad_prop")
      self.assertTrue(0, "Failed to handle bad prop (not a double)")
    except RuntimeError as e:
      self.assertTrue(re.search(r"[B,b]ad any[\ ,_]cast",str(e)))

  def testOxidationNumbers(self):
    # majority of tests are in the C++ layer.  These are just to make
    # sure the wrappers are working.
    m = Chem.MolFromSmiles("CO")
    rdMD.CalcOxidationNumbers(m)
    self.assertEqual(m.GetAtomWithIdx(0).GetIntProp('OxidationNumber'), -2)
    self.assertEqual(m.GetAtomWithIdx(1).GetIntProp('OxidationNumber'), -2)

    rdbase = environ["RDBASE"]
    ffile = Path(rdbase) / 'Code' / 'GraphMol' / 'MolStandardize' / 'test_data' / 'ferrocene.mol'
    ferrocene = Chem.MolFromMolFile(str(ffile))
    Chem.Kekulize(ferrocene)
    rdMD.CalcOxidationNumbers(ferrocene)
    self.assertEqual(ferrocene.GetAtomWithIdx(10).GetProp('OxidationNumber'), '2')


if __name__ == '__main__':
  unittest.main()



#!/usr/env/bin python
# -*- coding: utf-8 -*-
"""
unit testing for MolVS steps for PubChem Substances
tests include
molvs.standardize_smiles
Standardizer().normalize
Standardizer().disconnect_metals
Standardizer().reionize
molvs.standardize.canonicalize_tautomer_smiles
molvs.validate.Validator()
Standardizer().fragment_parent
"""
import gzip
import os.path
import unittest
from collections import namedtuple

import molvs
from molvs import Standardizer, validate

from rdkit import Chem, RDConfig

doLong = False
TestData = namedtuple('TestData', 'lineNo,smiles,mol,expected')


class TestCase(unittest.TestCase):
  dataPCS_standardize_smiles100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem',
                                                'MolStandardize', 'test_data',
                                                '100kPCS_standardize_sm.csv.gz')
  dataPCS_standardize_smiles1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                              'test_data', '1kPCS_standardize_sm.csv.gz')
  dataPCS_nomralized1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                      'test_data', '1kPCS_normalized.csv.gz')
  dataPCS_nomralized100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                        'test_data', '100kPCS_normalized.csv.gz')
  dataPCS_metal100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                   'test_data', '100kPCS_metals.csv.gz')
  dataPCS_metal1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize', 'test_data',
                                 '1kPCS_metals.csv.gz')
  dataPCS_reionize100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                      'test_data', '100kPCS_reionize.csv.gz')
  dataPCS_reionize1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                    'test_data', '1kPCS_reionize.csv.gz')
  dataPCS_fragment100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                      'test_data', '100kPCS_fragment.csv.gz')
  dataPCS_fragmnet1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                    'test_data', '1kPCS_fragment.csv.gz')
  dataPCS_tautomer100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                      'test_data', '100kPCS_tautomer.csv.gz')
  dataPCS_tautomer1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                    'test_data', '1kPCS_tautomer.csv.gz')
  dataPCS_validate100k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                      'test_data', '100kPCS_validate.csv.gz')
  dataPCS_validate1k = os.path.join(RDConfig.RDBaseDir, 'rdkit', 'Chem', 'MolStandardize',
                                    'test_data', '1kPCS_validate.csv.gz')

  @classmethod
  def readPCSdata(cls, datafile):
    for data in readPCStestData(datafile):
      yield data

  def testStandardizeSmShort(self):
    for data in self.readPCSdata(self.dataPCS_standardize_smiles1k):
      ss = molvs.standardize_smiles(data.smiles)
      self.assertEqual(ss, data.expected)

  def testStandardizeSmLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_standardize_smiles100k):
      try:
        ss = molvs.standardize_smiles(data.smiles)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS standardization failed for SMILES {data.smiles}')
      self.assertEqual(ss, data.expected)

  def testNormalizeShort(self):
    for data in self.readPCSdata(self.dataPCS_nomralized1k):
      n = Standardizer()
      nm = n.normalize(data.mol)
      ns = Chem.MolToSmiles(nm)
      self.assertEqual(ns, data.expected)

  def testNormalizeLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_nomralized100k):
      try:
        n = Standardizer()
        nm = n.normalize(data.mol)
        ns = Chem.MolToSmiles(nm)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(ns, data.expected)

  def testMetalShort(self):
    for data in self.readPCSdata(self.dataPCS_metal1k):
      n = Standardizer()
      nm = n.disconnect_metals(data.mol)
      ns = Chem.MolToSmiles(nm)
      self.assertEqual(ns, data.expected)

  def testMetalLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_metal100k):
      try:
        n = Standardizer()
        nm = n.disconnect_metals(data.mol)
        ns = Chem.MolToSmiles(nm)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(ns, data.expected)

  def testReionizeShort(self):
    for data in self.readPCSdata(self.dataPCS_reionize1k):
      n = Standardizer()
      nm = n.reionize(data.mol)
      ns = Chem.MolToSmiles(nm)
      self.assertEqual(ns, data.expected)

  def testReionizeLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_reionize100k):
      try:
        n = Standardizer()
        nm = n.reionize(data.mol)
        ns = Chem.MolToSmiles(nm)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(ns, data.expected)

  def testFragmentShort(self):
    for data in self.readPCSdata(self.dataPCS_fragmnet1k):
      s = Standardizer()
      frag = s.fragment_parent(data.mol)
      ns = Chem.MolToSmiles(frag)
      self.assertEqual(ns, data.expected)

  def testFragmentLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_fragment100k):
      try:
        s = Standardizer()
        frag = s.fragment_parent(data.mol)
        ns = Chem.MolToSmiles(frag)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(ns, data.expected)

  def testTautomerShort(self):
    for data in self.readPCSdata(self.dataPCS_tautomer1k):
      canon_taut = molvs.standardize.canonicalize_tautomer_smiles(data.smiles)
      self.assertEqual(canon_taut, data.expected)

  def testTautomerLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for data in self.readPCSdata(self.dataPCS_tautomer100k):
      try:
        canon_taut = molvs.standardize.canonicalize_tautomer_smiles(data.smiles)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(canon_taut, data.expected)

  def testValidateShort(self):
    for data in self.readPCSdata(self.dataPCS_validate1k):
      v = validate.Validator()
      stdout = v(data.mol)
      if stdout:
        stdout = ' '.join(stdout)
      else:
        stdout = str(stdout)
      self.assertEqual(stdout, data.expected)

  def testValidateLong(self):
    if not doLong:
      raise unittest.SkipTest('long test')
    for i, data in enumerate(self.readPCSdata(self.dataPCS_validate100k)):
      try:
        v = validate.Validator()
        stdout = v(data.mol)
        if stdout:
          stdout = ','.join(stdout)
        else:
          stdout = str(stdout)
        # print(i, stdout)
      except Exception:
        raise AssertionError(
          f'Line {data.lineNo}: MolVS normalization failed for SMILES {data.smiles}')
      self.assertEqual(stdout, data.expected)


def readPCStestData(filename):
  """ Read test data for MolVS from file """
  with gzip.open(filename, 'rt') as f:
    for lineNo, line in enumerate(f, 1):
      if line[0] == '#':
        continue
      smiles = line.strip().split(',')[0]
      expected = ",".join(line.strip().split(',')[1:])
      mol = Chem.MolFromSmiles(smiles)
      if not mol:
        raise AssertionError(f'molecule construction failed on line {lineNo}')
      yield TestData(lineNo, smiles, mol, expected)


if __name__ == '__main__':
  import argparse
  import sys

  parser = argparse.ArgumentParser()
  parser.add_argument('-l', default=False, action='store_true', dest='doLong')
  args = parser.parse_args()
  doLong = args.doLong

  # Remove the -l flag if present so that it isn't interpreted by unittest.main()
  if '-l' in sys.argv:
    sys.argv.remove('-l')
  unittest.main()



#
#  Copyright (C) 2001,2003  greg Landrum and Rational Discovery LLC
#
""" unit tests for the QuantTree implementation """

import io
import pickle
import unittest
from io import StringIO

from rdkit import RDConfig
from rdkit.ML.DecTree import BuildQuantTree
from rdkit.ML.DecTree.QuantTree import QuantTreeNode
from rdkit.TestRunner import redirect_stdout


def cmp(t1, t2):
  return (t1 < t2) * -1 or (t1 > t2) * 1


class TestCase(unittest.TestCase):

  def setUp(self):
    self.qTree1Name = RDConfig.RDCodeDir + '/ML/DecTree/test_data/QuantTree1.pkl'
    self.qTree2Name = RDConfig.RDCodeDir + '/ML/DecTree/test_data/QuantTree2.pkl'

  def _setupTree1(self):
    examples1 = [['p1', 0, 1, 0.1, 0], ['p2', 0, 0, 0.1, 1], ['p3', 0, 0, 1.1, 2],
                 ['p4', 0, 1, 1.1, 2], ['p5', 1, 0, 0.1, 2], ['p6', 1, 0, 1.1, 2],
                 ['p7', 1, 1, 0.1, 2], ['p8', 1, 1, 1.1, 0]]
    attrs = list(range(1, len(examples1[0]) - 1))
    nPossibleVals = [0, 2, 2, 0, 3]
    boundsPerVar = [0, 0, 0, 1, 0]

    self.t1 = BuildQuantTree.QuantTreeBoot(examples1, attrs, nPossibleVals, boundsPerVar)
    self.examples1 = examples1

  def _setupTree2(self):
    examples1 = [['p1', 0.1, 1, 0.1, 0], ['p2', 0.1, 0, 0.1, 1], ['p3', 0.1, 0, 1.1, 2],
                 ['p4', 0.1, 1, 1.1, 2], ['p5', 1.1, 0, 0.1, 2], ['p6', 1.1, 0, 1.1, 2],
                 ['p7', 1.1, 1, 0.1, 2], ['p8', 1.1, 1, 1.1, 0]]
    attrs = list(range(1, len(examples1[0]) - 1))
    nPossibleVals = [0, 0, 2, 0, 3]
    boundsPerVar = [0, 1, 0, 1, 0]

    self.t2 = BuildQuantTree.QuantTreeBoot(examples1, attrs, nPossibleVals, boundsPerVar)
    self.examples2 = examples1

  def _setupTree1a(self):
    examples1 = [['p1', 0, 1, 0.1, 4.0, 0], ['p2', 0, 0, 0.1, 4.1, 1], ['p3', 0, 0, 1.1, 4.2, 2],
                 ['p4', 0, 1, 1.1, 4.2, 2], ['p5', 1, 0, 0.1, 4.2, 2], ['p6', 1, 0, 1.1, 4.2, 2],
                 ['p7', 1, 1, 0.1, 4.2, 2], ['p8', 1, 1, 1.1, 4.0, 0]]
    attrs = list(range(1, len(examples1[0]) - 1))
    nPossibleVals = [0, 2, 2, 0, 0, 3]
    boundsPerVar = [0, 0, 0, 1, -1, 0]

    self.t1 = BuildQuantTree.QuantTreeBoot(examples1, attrs, nPossibleVals, boundsPerVar)
    self.examples1 = examples1

  def test0Cmp(self):
    # " testing tree comparisons "
    self._setupTree1()
    self._setupTree2()
    assert self.t1 == self.t1, 'self equals failed'
    assert self.t2 == self.t2, 'self equals failed'
    assert self.t1 != self.t2, 'not equals failed'

  def test1Tree(self):
    # " testing tree1 "
    self._setupTree1()
    with open(self.qTree1Name, 'r') as inTFile:
      buf = inTFile.read().replace('\r\n', '\n').encode('utf-8')
      inTFile.close()
    with io.BytesIO(buf) as inFile:
      t2 = pickle.load(inFile)
    assert self.t1 == t2, 'Incorrect tree generated. '

    self.assertIn('Var: 2 []', str(self.t1))
    self.assertEqual(self.t1.GetQuantBounds(), [])

  def test2Tree(self):
    # " testing tree2 "
    self._setupTree2()
    with open(self.qTree2Name, 'r') as inTFile:
      buf = inTFile.read().replace('\r\n', '\n').encode('utf-8')
      inTFile.close()
    with io.BytesIO(buf) as inFile:
      t2 = pickle.load(inFile)
    assert self.t2 == t2, 'Incorrect tree generated.'

  def test3Classify(self):
    # " testing classification "
    self._setupTree1()
    self._setupTree2()
    for i in range(len(self.examples1)):
      self.assertEqual(self.t1.ClassifyExample(self.examples1[i]), self.examples1[i][-1],
                       msg='examples1[%d] misclassified' % i)
    for i in range(len(self.examples2)):
      self.assertEqual(self.t2.ClassifyExample(self.examples2[i]), self.examples2[i][-1],
                       msg='examples2[%d] misclassified' % i)

  def test4UnusedVars(self):
    # " testing unused variables "
    self._setupTree1a()
    with open(self.qTree1Name, 'r') as inTFile:
      buf = inTFile.read().replace('\r\n', '\n').encode('utf-8')
      inTFile.close()
    with io.BytesIO(buf) as inFile:
      t2 = pickle.load(inFile)
    assert self.t1 == t2, 'Incorrect tree generated.'
    for i in range(len(self.examples1)):
      self.assertEqual(self.t1.ClassifyExample(self.examples1[i]), self.examples1[i][-1],
                       'examples1[%d] misclassified' % i)

  def test5Bug29(self):
    # """ a more extensive test of the cmp stuff using hand-built trees """
    import copy

    t1 = QuantTreeNode(None, 't1')
    t1.SetQuantBounds([1.])
    c1 = QuantTreeNode(t1, 'c1')
    c1.SetQuantBounds([2.])
    t1.AddChildNode(c1)
    c2 = QuantTreeNode(t1, 'c2')
    c2.SetQuantBounds([2.])
    t1.AddChildNode(c2)
    c11 = QuantTreeNode(c1, 'c11')
    c11.SetQuantBounds([3.])
    c1.AddChildNode(c11)
    c12 = QuantTreeNode(c1, 'c12')
    c12.SetQuantBounds([3.])
    c1.AddChildNode(c12)
    assert not cmp(t1, copy.deepcopy(t1)), 'self equality failed'

    t2 = QuantTreeNode(None, 't1')
    t2.SetQuantBounds([1.])
    c1 = QuantTreeNode(t2, 'c1')
    c1.SetQuantBounds([2.])
    t2.AddChildNode(c1)
    c2 = QuantTreeNode(t2, 'c2')
    c2.SetQuantBounds([2.])
    t2.AddChildNode(c2)
    c11 = QuantTreeNode(c1, 'c11')
    c11.SetQuantBounds([3.])
    c1.AddChildNode(c11)
    c12 = QuantTreeNode(c1, 'c12')
    c12.SetQuantBounds([3.00003])
    c1.AddChildNode(c12)
    assert cmp(t1, t2), 'inequality failed'

  def test6Bug29_2(self):
    # """ a more extensive test of the cmp stuff using pickled trees"""
    import os
    with open(os.path.join(RDConfig.RDCodeDir, 'ML', 'DecTree', 'test_data', 'CmpTree1.pkl'),
              'r') as t1TFile:
      buf = t1TFile.read().replace('\r\n', '\n').encode('utf-8')
      t1TFile.close()
    with io.BytesIO(buf) as t1File:
      t1 = pickle.load(t1File)
    with open(os.path.join(RDConfig.RDCodeDir, 'ML', 'DecTree', 'test_data', 'CmpTree2.pkl'),
              'r') as t2TFile:
      buf = t2TFile.read().replace('\r\n', '\n').encode('utf-8')
      t2TFile.close()
    with io.BytesIO(buf) as t2File:
      t2 = pickle.load(t2File)
    assert cmp(t1, t2), 'equality failed'

  def test7Recycle(self):
    # """ try recycling descriptors """
    examples1 = [
      [3, 0, 0],
      [3, 1, 1],
      [1, 0, 0],
      [0, 0, 1],
      [1, 1, 0],
    ]
    attrs = list(range(2))
    nPossibleVals = [2, 2, 2]
    boundsPerVar = [1, 0, 0]
    self.t1 = BuildQuantTree.QuantTreeBoot(examples1, attrs, nPossibleVals, boundsPerVar,
                                           recycleVars=1)
    assert self.t1.GetLabel() == 0, self.t1.GetLabel()
    assert self.t1.GetChildren()[0].GetLabel() == 1
    assert self.t1.GetChildren()[1].GetLabel() == 1
    assert self.t1.GetChildren()[1].GetChildren()[0].GetLabel() == 0
    assert self.t1.GetChildren()[1].GetChildren()[1].GetLabel() == 0

  def test8RandomForest(self):
    # """ try random forests descriptors """
    import random
    random.seed(23)
    nAttrs = 100
    nPts = 10
    examples = []
    for _ in range(nPts):
      descrs = [int(random.random() > 0.5) for _ in range(nAttrs)]
      act = sum(descrs) > nAttrs / 2
      examples.append(descrs + [act])
    attrs = list(range(nAttrs))
    nPossibleVals = [2] * (nAttrs + 1)
    boundsPerVar = [0] * nAttrs + [0]
    self.t1 = BuildQuantTree.QuantTreeBoot(examples, attrs, nPossibleVals, boundsPerVar, maxDepth=1,
                                           recycleVars=1, randomDescriptors=3)
    self.assertEqual(self.t1.GetLabel(), 49)
    self.assertEqual(self.t1.GetChildren()[0].GetLabel(), 3)
    self.assertEqual(self.t1.GetChildren()[1].GetLabel(), 54)

  def test_exampleCode(self):
    f = StringIO()
    with redirect_stdout(f):
      BuildQuantTree.TestTree()
    self.assertIn('Var: 2', f.getvalue())


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
#  Copyright (C) 2001-2008  greg Landrum and Rational Discovery LLC
#    All Rights Reserved
#
""" unit testing code for composite models

"""
import io
import pickle
import unittest

from rdkit import RDConfig
from rdkit.ML.Composite import Composite
from rdkit.ML.DecTree.DecTree import DecTreeNode as Node


class TestCase(unittest.TestCase):

  def setUp(self):
    with open(RDConfig.RDCodeDir + '/ML/Composite/test_data/ferro.pkl', 'r') as pklTF:
      buf = pklTF.read().replace('\r\n', '\n').encode('utf-8')
      pklTF.close()
    with io.BytesIO(buf) as pklF:
      self.examples = pickle.load(pklF)
    self.varNames = [
      'composition', 'max_atomic', 'has3d', 'has4d', 'has5d', 'elconc', 'atvol', 'isferro'
    ]
    self.qBounds = [[], [1.89, 3.53], [], [], [], [0.55, 0.73], [11.81, 14.52], []]
    self.nPoss = [0, 3, 2, 2, 2, 3, 3, 2]
    self.attrs = list(range(1, len(self.varNames) - 1))
    from rdkit.ML.Data import DataUtils
    DataUtils.InitRandomNumbers((23, 43))

  def testQuantize(self):
    # testing data quantization
    qBounds = [[], [1, 2, 3]]
    examples = [['foo', 0], ['foo', 1.5], ['foo', 5.5], ['foo', 2.5]]
    answers = [['foo', 0], ['foo', 1], ['foo', 3], ['foo', 2]]
    nPoss = [0, 4]
    composite = Composite.Composite()
    composite.SetQuantBounds(qBounds, nPoss)
    for i in range(len(examples)):
      qEx = composite.QuantizeExample(examples[i])
      self.assertEqual(qEx, answers[i])

  def testTreeGrow(self):
    # testing tree-based composite
    with open(RDConfig.RDCodeDir + '/ML/Composite/test_data/composite_base.pkl', 'r') as pklTF:
      buf = pklTF.read().replace('\r\n', '\n').encode('utf-8')
      pklTF.close()
    with io.BytesIO(buf) as pklF:
      self.refCompos = pickle.load(pklF)

    composite = Composite.Composite()
    composite._varNames = self.varNames
    composite.SetQuantBounds(self.qBounds, self.nPoss)
    from rdkit.ML.DecTree import CrossValidate
    driver = CrossValidate.CrossValidationDriver
    pruner = None
    composite.Grow(self.examples, self.attrs, [], buildDriver=driver, pruner=pruner, nTries=100,
                   silent=1)
    composite.AverageErrors()
    composite.SortModels(sortOnError=False)
    self.assertEqual(composite.countList, sorted(composite.countList))
    self.assertNotEqual(composite.errList, sorted(composite.errList))
    composite.SortModels()
    self.assertNotEqual(composite.countList, sorted(composite.countList))
    self.assertEqual(composite.errList, sorted(composite.errList))

    # with open(RDConfig.RDCodeDir+'/ML/Composite/test_data/composite_base.pkl','wb') as pklF:
    #   pickle.dump(composite,pklF)

    self.treeComposite = composite
    self.assertEqual(len(composite), len(self.refCompos))
    for i in range(len(composite)):
      t1, c1, e1 = composite[i]
      t2, c2, e2 = self.refCompos[i]
      self.assertEqual(e1, e2)
      # we used to check for equality here, but since there are redundant errors,
      #  that's non-trivial.
      # assert t1 == t2, 'tree mismatch'
      # assert c1 == c2, 'count mismatch'
    s = str(composite)
    self.assertIn('Composite', s)
    self.assertIn('Model', s)
    self.assertIn('error', s)

  def testErrorEstimate(self):
    # testing out-of-bag error estimates

    compos = Composite.Composite()
    compos.SetQuantBounds([(0.5, ), (0.5, ), (0.5, ), []], [2, 2, 2, 2])
    compos.SetDescriptorNames(('D0', 'D1', 'D2', 'Act'))
    compos.SetInputOrder(('ID', 'D0', 'D1', 'D2', 'Act'))
    data = [['A', 0, 0, 0, 0], ['B', 1, 0, 0, 1], ['C', 0, 1, 0, 0], ['D', 1, 1, 1, 1]]

    #
    #  Build and validate three simple trees:
    #
    t1 = Node(None, 'D0', 0)
    n = Node(t1, 'D1', 1)
    t1.AddChildNode(n)
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    n = Node(t1, 'D2', 2)
    t1.AddChildNode(n)
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    assert t1.ClassifyExample(data[0][1:]) == 0
    assert t1.ClassifyExample(data[1][1:]) == 1
    assert t1.ClassifyExample(data[2][1:]) == 1
    assert t1.ClassifyExample(data[3][1:]) == 0
    t1._trainIndices = (0, 1)
    compos.AddModel(t1, .5)

    t2 = Node(None, 'D1', 1)
    n = Node(t2, 'D0', 0)
    t2.AddChildNode(n)
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    n = Node(t2, 'D2', 2)
    t2.AddChildNode(n)
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    assert t2.ClassifyExample(data[0][1:]) == 0
    assert t2.ClassifyExample(data[1][1:]) == 1
    assert t2.ClassifyExample(data[2][1:]) == 0
    assert t2.ClassifyExample(data[3][1:]) == 1
    t2._trainIndices = (1, 2)
    compos.AddModel(t2, 0.0)

    t3 = Node(None, 'D0', 0)
    n = Node(t3, 'D2', 2)
    t3.AddChildNode(n)
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    n = Node(t3, 'D1', 1)
    t3.AddChildNode(n)
    n.AddChildNode(Node(n, '0', 0, isTerminal=1))
    n.AddChildNode(Node(n, '1', 1, isTerminal=1))
    assert t3.ClassifyExample(data[0][1:]) == 0
    assert t3.ClassifyExample(data[1][1:]) == 0
    assert t3.ClassifyExample(data[2][1:]) == 0
    assert t3.ClassifyExample(data[3][1:]) == 1
    t3._trainIndices = (2, 3)
    compos.AddModel(t3, 0.25)

    #
    #  validate the composite itself:
    #
    pred, conf = compos.ClassifyExample(data[0])
    assert pred == 0
    assert conf == 1.0
    pred, conf = compos.ClassifyExample(data[1])
    assert pred == 1
    assert conf == 2. / 3.
    pred, conf = compos.ClassifyExample(data[2])
    assert pred == 0
    assert conf == 2. / 3.
    pred, conf = compos.ClassifyExample(data[3])
    assert pred == 1
    assert conf == 2. / 3.

    self.assertEqual(compos.GetVoteDetails(), [0, 1, 1])
    self.assertEqual(compos.GetInputOrder(), [1, 2, 3, 4])

    #
    #  now test the out-of-bag calculation:
    #
    pred, conf = compos.ClassifyExample(data[0], onlyModels=(1, 2))
    assert pred == 0
    assert conf == 1.0
    pred, conf = compos.ClassifyExample(data[1], onlyModels=(2, ))
    assert pred == 0
    assert conf == 1.0
    pred, conf = compos.ClassifyExample(data[2], onlyModels=(0, ))
    assert pred == 1
    assert conf == 1.0
    pred, conf = compos.ClassifyExample(data[3], onlyModels=(0, 1))
    assert pred == 0
    assert conf == 0.5

    compos.ClearModelExamples()

  def test_exceptions(self):
    compos = Composite.Composite()
    compos.SetQuantBounds([(0.5, ), (0.5, ), (0.5, ), []], [2, 2, 2, 2])
    compos.SetDescriptorNames(('ID', 'D0', 'D1', 'D2', 'Act'))
    compos.SetInputOrder(('ID', 'D2', 'D1', 'D0', 'Act'))
    self.assertEqual(compos._mapOrder, [0, 3, 2, 1, 4])
    # Probes caught exception for ID
    compos.SetInputOrder(('D2', 'D1', 'D0', 'Act'))
    self.assertEqual(compos._mapOrder, [0, 2, 1, 0, 3])
    # Probes caught exception for Act
    compos.SetInputOrder(('ID', 'D2', 'D1', 'D0'))
    self.assertEqual(compos._mapOrder, [0, 3, 2, 1, -1])

    self.assertRaises(ValueError, compos.SetInputOrder, ('Unknown', 'D0'))


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



# $Id$
#
# Copyright (C) 2001-2006 greg Landrum and Rational Discovery LLC
#
#   @@ All Rights Reserved @@
#  This file is part of the RDKit.
#  The contents are covered by the terms of the BSD license
#  which is included in the file license.txt, found at the root
#  of the RDKit source tree.
#
""" unit testing code for the C++ BitVects
"""

import os
import pickle
import unittest

from rdkit.DataStructs import cDataStructs

klass = cDataStructs.SparseBitVect


def feq(n1, n2, tol=1e-4):
  return abs(n1 - n2) <= tol


def ieq(n1, n2):
  return abs(n1 - n2) == 0


class VectTests(object):

  def testSparseIdx(self):
    """ test indexing into SparseBitVects
    """
    v = self.klass(10)
    v[0] = 1
    v[2] = 1
    v[9] = 1

    with self.assertRaisesRegex(IndexError, ""):
      v[10] = 1

    assert v[0] == 1, 'bad bit'
    assert v[1] == 0, 'bad bit'
    assert v[2] == 1, 'bad bit'
    assert v[9] == 1, 'bad bit'
    assert v[-1] == 1, 'bad bit'
    assert v[-2] == 0, 'bad bit'

    with self.assertRaisesRegex(IndexError, ""):
      _ = v[10]

  def testSparseBitGet(self):
    """ test operations to get sparse bits
    """
    v = self.klass(10)
    v[0] = 1
    v[2] = 1
    v[6] = 1
    assert len(v) == 10, 'len(SparseBitVect) failed'
    assert v.GetNumOnBits() == 3, 'NumOnBits failed'
    assert tuple(v.GetOnBits()) == (0, 2, 6), 'GetOnBits failed'

  def testSparseBitOps(self):
    """ test bit operations on SparseBitVects
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    assert tuple((v1 & v2).GetOnBits()) == (0, 6), 'binary & failed'
    assert tuple((v1 & v2).GetOnBits()) == (0, 6), 'binary & failed'
    assert tuple((v1 | v2).GetOnBits()) == (0, 2, 3, 6), 'binary | failed'
    assert tuple((v1 ^ v2).GetOnBits()) == (2, 3), 'binary ^ failed'

  def testTanimotoSim(self):
    """ test Tanimoto Similarity measure
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = self.klass(10)
    v3[1] = 1
    v3[4] = 1
    v3[8] = 1
    assert feq(cDataStructs.TanimotoSimilarity(v1, v1), 1.0), 'bad v1,v1 TanimotoSimilarity'
    assert feq(cDataStructs.TanimotoSimilarity(v2, v2), 1.0), 'bad v2,v2 TanimotoSimilarity'
    assert feq(cDataStructs.TanimotoSimilarity(v1, v2), 0.5), 'bad v1,v2 TanimotoSimilarity'
    assert feq(cDataStructs.TanimotoSimilarity(v2, v1), 0.5), 'bad v2,v1 TanimotoSimilarity'
    assert feq(cDataStructs.TanimotoSimilarity(v1, v3), 0.0), 'bad v1,v3 TanimotoSimilarity'
    assert feq(cDataStructs.TanimotoSimilarity(v2, v3), 0.0), 'bad v2,v3 TanimotoSimilarity'

  def testOnBitSim(self):
    """ test On Bit Similarity measure
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = self.klass(10)
    v3[1] = 1
    v3[4] = 1
    v3[8] = 1
    assert feq(cDataStructs.OnBitSimilarity(v1, v1), 1.0), 'bad v1,v1 OnBitSimilarity'
    assert feq(cDataStructs.OnBitSimilarity(v2, v2), 1.0), 'bad v2,v2 OnBitSimilarity'
    assert feq(cDataStructs.OnBitSimilarity(v1, v2), 0.5), 'bad v1,v2 OnBitSimilarity'
    assert feq(cDataStructs.OnBitSimilarity(v2, v1), 0.5), 'bad v2,v1 OnBitSimilarity'
    assert feq(cDataStructs.OnBitSimilarity(v1, v3), 0.0), 'bad v1,v3 OnBitSimilarity'
    assert feq(cDataStructs.OnBitSimilarity(v2, v3), 0.0), 'bad v2,v3 OnBitSimilarity'

  def testNumBitsInCommon(self):
    """ test calculation of Number of Bits in Common
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = self.klass(10)
    v3[1] = 1
    v3[4] = 1
    v3[8] = 1
    assert ieq(cDataStructs.NumBitsInCommon(v1, v1), 10), 'bad v1,v1 NumBitsInCommon'
    assert ieq(cDataStructs.NumBitsInCommon(v2, v2), 10), 'bad v2,v2 NumBitsInCommon'
    assert ieq(cDataStructs.NumBitsInCommon(v1, v2), 8), 'bad v1,v2 NumBitsInCommon'
    assert ieq(cDataStructs.NumBitsInCommon(v2, v1), 8), 'bad v2,v1 NumBitsInCommon'
    assert ieq(cDataStructs.NumBitsInCommon(v1, v3), 4), 'bad v1,v3 NumBitsInCommon'
    assert ieq(cDataStructs.NumBitsInCommon(v2, v3), 4), 'bad v2,v3 NumBitsInCommon'

  def testAllBitSim(self):
    """ test All Bit Similarity measure
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = self.klass(10)
    v3[1] = 1
    v3[4] = 1
    v3[8] = 1
    assert feq(cDataStructs.AllBitSimilarity(v1, v1), 1.0), 'bad v1,v1 AllBitSimilarity'
    assert feq(cDataStructs.AllBitSimilarity(v2, v2), 1.0), 'bad v2,v2 AllBitSimilarity'
    assert feq(cDataStructs.AllBitSimilarity(v1, v2), 0.8), 'bad v1,v2 AllBitSimilarity'
    assert feq(cDataStructs.AllBitSimilarity(v2, v1), 0.8), 'bad v2,v1 AllBitSimilarity'
    assert feq(cDataStructs.AllBitSimilarity(v1, v3), 0.4), 'bad v1,v3 AllBitSimilarity'
    assert feq(cDataStructs.AllBitSimilarity(v2, v3), 0.4), 'bad v2,v3 AllBitSimilarity'

  def testStringOps(self):
    """ test serialization operations
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    s = v1.ToBinary()
    v2 = self.klass(s)
    assert tuple(v2.GetOnBits()) == tuple(v1.GetOnBits()), 'To/From string failed'

  def testOnBitsInCommon(self):
    """ test OnBitsInCommon
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = cDataStructs.OnBitsInCommon(v1, v2)
    assert tuple(v3) == (0, 6), 'bad on bits in common'

  def testOffBitsInCommon(self):
    """ test OffBitsInCommon
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    v3 = cDataStructs.OffBitsInCommon(v1, v2)
    assert tuple(v3) == (1, 4, 5, 7, 8, 9), 'bad off bits in common'

  def testOnBitProjSimilarity(self):
    """ test OnBitProjSimilarity
    """
    v1 = self.klass(10)
    v1[1] = 1
    v1[2] = 1
    v1[3] = 1
    v2 = self.klass(10)
    v2[2] = 1
    v2[3] = 1
    res = cDataStructs.OnBitProjSimilarity(v1, v2)
    assert feq(res[0], 0.666667), 'bad 1st OnBitsProjSimilarity'
    assert feq(res[1], 1.0), 'bad 2nd OnBitsProjSimilarity'
    res = cDataStructs.OnBitProjSimilarity(v2, v1)
    assert feq(res[1], 0.666667), 'bad 1st OnBitsProjSimilarity'
    assert feq(res[0], 1.0), 'bad 2nd OnBitsProjSimilarity'

  def testOffBitProjSimilarity(self):
    """ test OffBitProjSimilarity
    """
    v1 = self.klass(10)
    v1[1] = 1
    v1[2] = 1
    v1[3] = 1
    v2 = self.klass(10)
    v2[2] = 1
    v2[3] = 1
    res = cDataStructs.OffBitProjSimilarity(v1, v2)
    assert feq(res[0], 1.0), 'bad 1st OffBitsProjSimilarity'
    assert feq(res[1], 0.875), 'bad 2nd OffBitsProjSimilarity'
    res = cDataStructs.OffBitProjSimilarity(v2, v1)
    assert feq(res[1], 1.0), 'bad 1st OffBitsProjSimilarity'
    assert feq(res[0], 0.875), 'bad 2nd OffBitsProjSimilarity'

  def testPkl(self):
    # Test pickling
    v1 = self.klass(10)
    v1[1] = 1
    v1[2] = 1
    v1[3] = 1
    pklName = 'foo.pkl'
    outF = open(pklName, 'wb+')
    pickle.dump(v1, outF)
    outF.close()
    inF = open(pklName, 'rb')
    v2 = pickle.load(inF)
    inF.close()
    os.unlink(pklName)
    assert tuple(v1.GetOnBits()) == tuple(v2.GetOnBits()), 'pkl failed'

  def testFingerprints(self):
    # Test parsing Daylight fingerprints
    # actual daylight output:
    rawD = """
0,Cc1n[nH]c(=O)nc1N,.b+HHa.EgU6+ibEIr89.CpX0g8FZiXH+R0+Ps.mr6tg.2
1,Cc1n[nH]c(=O)[nH]c1=O,.b7HEa..ccc+gWEIr89.8lV8gOF3aXFFR.+Ps.mZ6lg.2
2,Cc1nnc(NN)nc1O,.H+nHq2EcY09y5EIr9e.8p50h0NgiWGNx4+Hm+Gbslw.2
3,Cc1nnc(N)nc1C,.1.HHa..cUI6i5E2rO8.Op10d0NoiWGVx.+Hm.Gb6lo.2
"""
    dists = """0,0,1.000000
0,1,0.788991
0,2,0.677165
0,3,0.686957
1,1,1.000000
1,2,0.578125
1,3,0.591304
2,2,1.000000
2,3,0.732759
3,3,1.000000
"""

    fps = []
    for line in rawD.split('\n'):
      if line:
        sbv = self.klass(256)
        _, _, fp = line.split(',')
        cDataStructs.InitFromDaylightString(sbv, fp)
        fps.append(sbv)

    ds = dists.split('\n')
    whichd = 0
    for i in range(len(fps)):
      for j in range(i, len(fps)):
        idx1, idx2, tgt = ds[whichd].split(',')
        whichd += 1
        tgt = float(tgt)
        dist = cDataStructs.TanimotoSimilarity(fps[i], fps[j])
        assert feq(tgt, dist), 'tanimoto between fps %d and %d failed' % (int(idx1), int(idx2))

  def testFold(self):
    """ test folding fingerprints
    """
    v1 = self.klass(16)
    v1[1] = 1
    v1[12] = 1
    v1[9] = 1
    v2 = cDataStructs.FoldFingerprint(v1)  # check fold with no args
    assert v1.GetNumBits() / 2 == v2.GetNumBits(), 'bad num bits post folding'
    v2 = cDataStructs.FoldFingerprint(v1, 2)  # check fold with arg

    assert v1.GetNumBits() / 2 == v2.GetNumBits(), 'bad num bits post folding'

    v2 = cDataStructs.FoldFingerprint(v1, 4)
    assert v1.GetNumBits() / 4 == v2.GetNumBits(), 'bad num bits post folding'

  def testOtherSims(self):
    """ test other similarity measures
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    assert feq(cDataStructs.CosineSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.KulczynskiSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.DiceSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.SokalSimilarity(v1, v2), .3333)
    assert feq(cDataStructs.McConnaugheySimilarity(v1, v2), .3333)
    assert feq(cDataStructs.AsymmetricSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.BraunBlanquetSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.RusselSimilarity(v1, v2), .2000)
    assert feq(cDataStructs.RogotGoldbergSimilarity(v1, v2), .7619)

  def testQuickSims(self):
    """ the asymmetric similarity stuff (bv,pkl)
    """
    v1 = self.klass(10)
    v1[0] = 1
    v1[2] = 1
    v1[6] = 1
    v2 = self.klass(10)
    v2[0] = 1
    v2[3] = 1
    v2[6] = 1
    pkl = v2.ToBinary()
    v2 = pkl
    assert feq(cDataStructs.CosineSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.KulczynskiSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.DiceSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.SokalSimilarity(v1, v2), .3333)
    assert feq(cDataStructs.McConnaugheySimilarity(v1, v2), .3333)
    assert feq(cDataStructs.AsymmetricSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.BraunBlanquetSimilarity(v1, v2), .6667)
    assert feq(cDataStructs.RusselSimilarity(v1, v2), .2000)
    assert feq(cDataStructs.RogotGoldbergSimilarity(v1, v2), .7619)


class SparseBitVectTests(VectTests, unittest.TestCase):
  klass = cDataStructs.SparseBitVect


class ExplicitTestCase(VectTests, unittest.TestCase):
  klass = cDataStructs.ExplicitBitVect


if __name__ == '__main__':  # pragma: nocover
  unittest.main()



#
#  Copyright (c) 2010, Novartis Institutes for BioMedical Research Inc.
#  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above
#       copyright notice, this list of conditions and the following
#       disclaimer in the documentation and/or other materials provided
#       with the distribution.
#     * Neither the name of Novartis Institutes for BioMedical Research Inc.
#       nor the names of its contributors may be used to endorse or promote
#       products derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Created by Greg Landrum October 2006
#

import os.path
import unittest

from rdkit import Chem, RDConfig
from rdkit.Chem import FunctionalGroups


class TestCase(unittest.TestCase):

  def test1Basics(self):
    txt = """
AcidChloride\tC(=O)Cl\tAcid Chloride
  AcidChloride.Benzoyl\tC(=O)(Cl)c1ccccc1\tBenzoyl
Amine\tN\tAmine
  Amine.Primary\t[N;H2]\tPrimary
    Amine.Primary.Aromatic\t[N;H2][a]\tPrimary Aromatic
  Amine.Aromatic\tN[a]\tAromatic
"""
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 2)
    self.assertEqual(len(hierarchy[0]), 2)
    self.assertEqual(len(hierarchy[1]), 4)
    self.assertEqual(hierarchy[0].name, 'Acid Chloride')
    self.assertEqual(hierarchy[0].children[0].name, 'Benzoyl')
    self.assertEqual(hierarchy[0].label, 'AcidChloride')
    self.assertEqual(hierarchy[0].rxnSmarts, '')
    m = Chem.MolFromSmiles('ClC(=O)CCCNc1ccccc1')
    fp = FunctionalGroups.CreateMolFingerprint(m, hierarchy)
    self.assertEqual(fp, [1, 0, 1, 0, 0, 1])

    m = Chem.MolFromSmiles('OC(=O)CCC')
    fp = FunctionalGroups.CreateMolFingerprint(m, hierarchy)
    self.assertEqual(fp, [0, 0, 0, 0, 0, 0])

    # make sure we get the same hierarchy on the second call:
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 2)
    self.assertEqual(len(hierarchy[0]), 2)
    self.assertEqual(len(hierarchy[1]), 4)
    self.assertEqual(hierarchy[0].name, 'Acid Chloride')
    self.assertEqual(hierarchy[0].children[0].name, 'Benzoyl')
    self.assertEqual(hierarchy[0].label, 'AcidChloride')
    self.assertEqual(hierarchy[0].rxnSmarts, '')

    # if we edit this hierarchy it doesn't affect the global one:
    hierarchy.pop(0)
    self.assertEqual(len(hierarchy[0]), 4)
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 2)
    self.assertEqual(len(hierarchy[0]), 2)
    self.assertEqual(len(hierarchy[1]), 4)
    self.assertEqual(hierarchy[0].name, 'Acid Chloride')
    self.assertEqual(hierarchy[0].children[0].name, 'Benzoyl')
    self.assertEqual(hierarchy[0].label, 'AcidChloride')
    self.assertEqual(hierarchy[0].rxnSmarts, '')

    # and if we edit the global one and don't force, we get the edited one:
    FunctionalGroups.hierarchy.pop(0)
    self.assertEqual(len(FunctionalGroups.hierarchy[0]), 4)
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 1)
    self.assertEqual(len(hierarchy[0]), 4)

    # but a force gets us back:
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt, force=True)
    self.assertEqual(len(hierarchy), 2)
    self.assertEqual(len(hierarchy[0]), 2)
    self.assertEqual(len(hierarchy[1]), 4)

  def test2Comments(self):
    txt = """
AcidChloride\tC(=O)Cl\tAcid Chloride
  AcidChloride.Benzoyl\tC(=O)(Cl)c1ccccc1\tBenzoyl
Amine\tN\tAmine
  Amine.Primary\t[N;H2]\tPrimary
    //Amine.Primary.Aromatic\t[N;H2][a]\tPrimary Aromatic
  Amine.Aromatic\tN[a]\tAromatic
"""
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 2)
    self.assertEqual(len(hierarchy[0]), 2)
    self.assertEqual(len(hierarchy[1]), 3)

  def test3Reactions(self):
    txt = """BoronicAcid\t[$(B-!@[#6])](O)(O)\tBoronic Acid\t[#6:1][B:2]([O:3])[O:4]>>[#6:1].[B:2]([O:3])[O:4]
  BoronicAcid.Aromatic\t[$(B-!@c)](O)(O)\tAromatic\t[c:1][B:2]([O:3])[O:4]>>[c:1].[B:2]([O:3])[O:4]
  BoronicAcid.Aliphatic\t[$(B-!@C)](O)(O)\tAliphatic\t[C:1][B:2]([O:3])[O:4]>>[C:1].[B:2]([O:3])[O:4]
  """
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy(data=txt)
    self.assertTrue(hierarchy)
    self.assertEqual(len(hierarchy), 1)
    self.assertEqual(len(hierarchy[0].children), 2)
    self.assertNotEqual(hierarchy[0].rxnSmarts, '')
    self.assertNotEqual(hierarchy[0].children[0].rxnSmarts, '')

  def test4Hs(self):
    hierarchy = FunctionalGroups.BuildFuncGroupHierarchy()

    inName = os.path.join(RDConfig.RDCodeDir, 'Chem', 'test_data', 'NCI_5K_TPSA.csv')
    with open(inName, 'r') as inF:
      ms = [Chem.MolFromSmiles(x.split(',')[0]) for x in inF if x[0] != '#']
    for m in ms:
      mh = Chem.AddHs(m)
      fp = FunctionalGroups.CreateMolFingerprint(m, hierarchy)
      fph = FunctionalGroups.CreateMolFingerprint(mh, hierarchy)
      if fp != fph:
        print(Chem.MolToSmiles(m))
        print(fp.ToBitString())
        print(fph.ToBitString())
      self.assertEqual(fp, fph)


if __name__ == '__main__':
  unittest.main()
